#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
:package: eWRT.ws.wot

:author: Heinz-Peter Lang

Client for accessing the Reputation API for Web of Trust

see: `WOT API Documentation <http://www.mywot.com/wiki/API#Documentation:_Reputation_API>`_
'''
import unittest
import json

from six import string_types
from urllib import urlencode, quote
from urlparse import urlparse

from eWRT.access.http import Retrieve


SERVICE_URL = 'http://api.mywot.com/0.4/public_link_json2?hosts=%(hosts)s&key=%(api_key)s'
WOT_LINK = 'http://www.mywot.com/en/scorecard/%s'
MAX_HOSTS = 100

MAPPING = {'0': 'trustworthiness',
           '1': 'vendor_reliability',
           '2': 'privacy',
           '4': 'child_safety',
           'target': 'target'}

class WebOfTrust(object):
    
    def __init__(self, api_key, service_url=SERVICE_URL):
        self.api_key = api_key
        self.service_url = service_url
        self.retrieve = Retrieve('eWRT.ws.wot')
        
    def get_reputation(self, hosts): 
        query={'hosts': self._encode_hosts(hosts),
               'api_key': self.api_key}
        
        urlObj = self.retrieve.open(self.service_url % query)
        
        if not urlObj:
            raise Exception('got no result')
        
        return self._format_result(json.loads(urlObj.read())) 
        
    @classmethod
    def _encode_hosts(cls, hosts):
        ''' 
        >>> WebOfTrust._encode_hosts(['http://wu.ac.at', 'https://wu.ac.at'])
        'wu.ac.at/'
        >>> WebOfTrust._encode_hosts(['wu.ac.at', 'https://modul.ac.at/'])
        'wu.ac.at/modul.ac.at/'
        '''
        if isinstance(hosts, string_types):
            hosts = [hosts]
        
        selected_hosts = []
        
        for host in hosts:
            
            if not host.startswith('http'):
                host = 'http://%s' % host
            netloc = '%s/' % quote(urlparse(host).netloc)
            
            if not netloc in selected_hosts: 
                selected_hosts.append(netloc)
        
        assert len(hosts) <= MAX_HOSTS, 'too many hosts (max: %s)!' % MAX_HOSTS
        return ''.join(selected_hosts)
    
    @classmethod
    def _encode_url(cls, service_url, query):
        ''' encodes the url '''
        return service_url % query

    @classmethod
    def _format_result(cls, data):
        '''
        Formats the result using MAPPING. The components for the reputation 
        provide the reputation and confidence. See WOT Developer API for 
        details
        '''
        result = {}
        for host, reputation in data.iteritems():
            r = {}
            for attr_name, new_attr_name in MAPPING.iteritems():
                if attr_name in reputation:
                    r[new_attr_name] = reputation[attr_name]
            r['wot_link'] = WOT_LINK % r['target']
            result[host] = r
            
        return result

class TestWOT(unittest.TestCase):
    
    def test_format_result(self):
        data = {u'diepresse.com': {u'1': [93, 54], u'0': [92, 57], 
                                   u'2': [93, 54], u'target': u'diepresse.com', 
                                   u'4': [93, 54]}, 
                u'derstandard.at': {u'1': [93, 60], u'0': [93, 62], 
                                    u'2': [93, 61], u'target': u'derstandard.at', 
                                    u'4': [93, 60]}}
        for details in WebOfTrust._format_result(data).itervalues():
            assert all(attr in details for attr in MAPPING.values())

        
if __name__ == '__main__':
    import doctest
    doctest.testmod()
    unittest.main()


#!/usr/bin/env python

''' @package eWRT.util.profile
    google like profiling :)

    @warning
    this library is still a draft and might change considerable
'''

# (C)opyrights 2008-2015 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__copyright__ = "GPL"

import __main__
import cProfile
import pstats
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


def profile(fn, logfile='profile.awi'):
    ''' profile function '''
    prof = cProfile.Profile()
    is_unittest = __main__.__file__.endswith('nosetests') or \
        __main__.__file__.endswith('py.test') or \
        __main__.__file__.endswith('py.test-3')

    if is_unittest:
        prof = prof.runctx("{}()".format(fn.__name__), globals(), locals())
    else:
        prof = prof.runctx("{}()".format(fn.__name__),
                           __main__.__dict__, __main__.__dict__)

    stream = StringIO()
    stats = pstats.Stats(prof, stream=stream)
    stats.sort_stats('time')  # Or cumulative
    stats.print_stats(80)  # 80 = how many to print

    # The rest is optional.
    stats.print_callees()
    stats.print_callers()
    open(logfile, 'w').write(stream.getvalue())

# ----------------------------------------------------------------------------
# unit testing
# ----------------------------------------------------------------------------


def profileFunction():
    ''' function to profile '''
    for a in range(1000):
        for b in range(1000):
            c = a * b

    return c


def testProfile():
    ''' test the profiling '''
    profile(profileFunction)



#!/usr/bin/env python

""" @package eWRT.ws.amazon
    provides access to amazon data """

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


__revision__ = "$Revision$"


import base64
import hashlib
import hmac
import urllib
import time
import logging

from urllib import quote
from xml.parsers.expat import ParserCreate
from time import strftime

from eWRT.access.http import Retrieve
from eWRT.config import AMAZON_ACCESS_KEY, AMAZON_LOCATIONS, AMAZON_DEBUG_FILE


# time to wait after an error in seconds
ERROR_SLEEP_TIME = 30

BROWSE_NODE_ID = {'book': '283155',
                  'dvd': '130',
                  'music': '5174'
                  }

AWS_ACCESS_KEY_ID = {'AWSAccessKeyId': AMAZON_ACCESS_KEY}


class ResultList(object):
    """ converts xml results into a list of dictionaries """

    def __init__(self, targetPath, hunt=()):

        assert(isinstance(targetPath, str))

        self.p = ParserCreate()
        self.p.StartElementHandler = self.startElement
        self.p.EndElementHandler = self.endElement
        self.p.CharacterDataHandler = self.charData

        self.xmlResult = []
        self.targetPath = targetPath
        self.path = []

        self.hunt = hunt
        self.huntValue = {}

    def startElement(self, element, attrs):
        """ start Element handler """
        self.path.append(element)
        if self.targetPath == "/".join(self.path):
            self.xmlResult.append({})

    def charData(self, data):
        """ adds data to the dictionary """
        currentElement = self.path[-1]
        if self.targetPath in "/".join(self.path):
            try:
                self.xmlResult[-1][currentElement] = self.xmlResult[-1].get(
                    currentElement, "") + data.encode("utf8")
            except IndexError:
                raise "Given Search Path '%s' is not a Sub-Path of '%s'\n" % (
                    "/".join(self.path), self.targetPath)

        if currentElement in self.hunt:
            self.huntValue[currentElement] = data.encode("utf8")

    def endElement(self, element):
        assert(self.path)
        self.path.pop()

    def parse(self, data):
        """ parses xml data """
        self.p.Parse(data)

    def parseNew(self, data):
        self.path = []
        self.p.Parse(data)

    def test(self):
        self.parse(open("a.xml").read())
        print self.xmlResult


class AmazonWS(object):
    """ This class provides low level amazon web service access """

    def __init__(self, location='us', key=None):
        """ init """
        assert (location in AMAZON_LOCATIONS)
        self.retrieve = Retrieve(self.__class__.__name__)
        self.wsBase = AMAZON_LOCATIONS[location]
        self.accessKey = key or AMAZON_ACCESS_KEY
        self.amazon_url = AmazonUrl()

    def generateWsUrl(self, arguments):
        """ generates a valid amazon webservice request url """
        argList = ["%s&SubscriptionId=%s" % (
            self.wsBase, self.accessKey)] + ["%s=%s" % (k, quote(v)) for k, v in arguments.items()]
        return "&".join(argList)

    def generateSignedWsUrl(self, **arguments):
        """ generates a valid amazon webservice request url """
        #argList = [ "%s&SubscriptionId=%s" % (self.wsBase, self.accessKey) ] + [ "%s=%s" % (k,quote(v)) for k,v in arguments.items() ]
        # return "&".join(argList)
        return self.amazon_url.get_request_url(arguments)

    def query(self, arguments):
        """ retrieves a result from amazon webservice """
        url = self.generateWsUrl(arguments)

        done = False
        while not done:
            try:
                f = self.retrieve.open(url)
                res = f.read()
                self._write_debug_data(res)
                f.close()
                done = True
            except ValueError:
                logging.warning(
                    "Exception webservice query - waiting for %d seconds...\n" % ERROR_SLEEP_TIME)
                time.sleep(ERROR_SLEEP_TIME)
        return res

        @staticmethod
        def _write_debug_data(data):
            """ writes the given data to the debug file, if specified """
            if not AMAZON_DEBUG_FILE:
                return

            d = open(AMAZON_DEBUG_FILE, "a")
            d.write(data)
            d.close()

    def searchItem(self, searchIndex='Books', **param):
        """ searches an item in the amazon product repository """
        arguments = {'Operation': 'ItemSearch',
                     'SearchIndex': searchIndex,
                     'BrowseNode': '1000',
                     'Sort': 'salesrank',
                     'ResponseGroup': 'SalesRank,Small'}

        arguments.update(param)
        return self.query(arguments)

    def queryReview(self, itemId, **param):
        """ queries customers reviews to the selected Item """
        arguments = {'Operation': 'ItemLookup',
                     'ResponseGroup': 'Reviews',
                     'ItemId': itemId}
        arguments.update(param)
        return self.query(arguments)

        def newReleases(self, **param):
            """ returns a list of asins of new releases """
            arguments = {'Operation': 'BrowseNodeLookup',
                         'ResponseGroup': 'NewReleases',
                         'Marketplace': 'us'}

            arguments.update(param)
            return self.query(arguments)

        def itemAttributes(self, item_id, **param):
            """ returns all item attribues """
            arguments = {'Operation': 'ItemLookup',
                         'ItemId': item_id,
                         'IdType': 'ASIN',
                         'ResponseGroup': 'ItemAttributes,SalesRank'}
            arguments.update(param)
            return self.query(arguments)


class AmazonUrl(object):

    def timestamp(self):
        """ determine the current timestamp """
        current_timestamp = strftime('%Y-%m-%dT%H:%M:%SZ')
        return {'Timestamp': current_timestamp}

    def get_signature(self, msg):
        """ apply hmac with sha256 to the msg using key """
        hmac_sha256_alg = hmac.new(self.access_key, msg, hashlib.sha256)
        signature = hmac_sha256_alg.digest()
        signature = base64.encodestring(signature)
        return signature

    def get_msgs(self, params):
        """ determine the sig_msg (i.e. the message to be hmac encoded) and the
        url_msg (i.e. the message to be sent to amazon) """

        GET = 'GET'
        ECS = 'ecs.amazonaws.com'
        ONCA = '/onca/xml'

        params.update(self.timestamp())
        params.update(AWS_ACCESS_KEY_ID)
        url_encoded = urllib.urlencode(params)

        msg = [entry for entry in url_encoded.split('&')]
        msg.sort()  # amazon also calculates signature from sorted entries
        msg = '&'.join(msg)

        sig_msg = '\n'.join([GET, ECS, ONCA, msg])
        url_msg = '%s%s%s?%s' % ('http://', ECS, ONCA, msg)
        return url_msg, sig_msg

    def get_request_url(self, params):
        """ generate a url containing signature and timestamp with the given
        parameters """
        url_msg, sig_msg = self.get_msgs(params)
        signature = self.get_signature(sig_msg)
        request_url = "%s&%s" % (
            url_msg, urllib.urlencode({'Signature': signature}))
        return request_url.replace('%0A', '')


if __name__ == "__main__":
    url_generator = AmazonWS()
    url = url_generator.generateSignedWsUrl(Operation='BrowseNodeLookup', Service='AWSECommerceService',
                                            ResponseGroup='NewReleases', Marketplace='us', BrowseNodeId='281052')
    print url



#!/usr/bin/env python

''' @package eWRT.access.http
    provides access to resources using http '''
from __future__ import print_function


# (C)opyrights 2008-2015 by Albert Weichselbraun <albert@weblyzard.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

try:
    # urllib2 is merged into urllib in python3 (SV)
    import urllib.request as urllib2
except:
    import urllib2  # python2

from eWRT.config import (USER_AGENT, DEFAULT_WEB_REQUEST_SLEEP_TIME,
                         PROXY_SERVER)

#USER_AGENT = 'eWRT Version/0.1; Module %s +http://p.semanticlab.net/eWRT'
#DEFAULT_WEB_REQUEST_SLEEP_TIME = 1
#PROXY_SERVER = None

try:
    from urllib.parse import urlsplit, urlunsplit  # porting to python 3.4 (SV)
except:
    from urlparse import urlsplit, urlunsplit  # python2

import time
import io

from gzip import GzipFile
from random import randint


# logging
import logging
log = logging.getLogger(__name__)

RETRY_WAIT_TIME_RANGE = (2, 10)              # in seconds
# error codes which might trigger a retry:
HTTP_TEMPORARY_ERROR_CODES = (500, 503, 504)

# set default socket timeout (otherwise urllib might hang!)
from socket import setdefaulttimeout
DEFAULT_TIMEOUT = 60


def getHostName(x): return "://".join(urlsplit(x)[:2])


class Retrieve(object):
    ''' @class Retrieve
        retrieves URLs using HTTP

        .. remarks:
           this class supports transparent
           - authentication and
           - compression
           - support for the context protocol (python)
           - automatic throttling support

        @warning
        There are certain urls such as
        http://www.mfsa.com.mt/insguide/english/glossarysearch.jsp?letter=all
        which are _not_ handled correctly by the underlying urllib2 library(!)
        Please use urllib in such cases.

    '''

    __slots__ = ('module', 'sleep_time', 'last_access_time', 'user_agent',
                 '_supported_http_authentification_methods')

    def __init__(self, module, sleep_time=DEFAULT_WEB_REQUEST_SLEEP_TIME,
                 user_agent=USER_AGENT, default_timeout=DEFAULT_TIMEOUT):
        setdefaulttimeout(default_timeout)
        self.module = module
        self.sleep_time = sleep_time
        self.last_access_time = 0

        self._supported_http_authentification_methods = {
            'basic': Retrieve._getHTTPBasicAuthOpener,
            'digest': Retrieve._getHTTPDigestAuthOpener}

        self.user_agent = user_agent % self.module \
            if "%s" in user_agent else user_agent

    def open(self, url, data=None, headers={}, user=None, pwd=None, retry=0,
             authentification_method="basic", accept_gzip=True,
             head_only=False):
        ''' Opens an URL and returns the matching file object
            @param[in] url
            @param[in] data    optional data to submit
            @param[in] headers a dictionary of optional headers
            @param[in] user    optional user name
            @param[in] pwd     optional password
            @param[in] retry   number of retries in case of an temporary error
            @param[in] authentification_method the used authentification_method
                        ('basic'*, 'digest')
            @param[in] accept_gzip flag to change the accepted encoding, gzip
                        or not
            @param[in] head_only   if True: only execute a HEAD request
            @returns a file object for reading the url
        '''
        auth_handler = self._supported_http_authentification_methods[
            authentification_method]
        urlObj = None
        tries = 0
        while not urlObj:
            request = urllib2.Request(url, data, headers)

            if head_only:
                request.get_method = lambda: 'HEAD'

            request.add_header('User-Agent', self.user_agent)

            if accept_gzip:
                request.add_header('Accept-encoding', 'gzip')

            self._throttle()

            opener = []
            if PROXY_SERVER:
                opener.append(urllib2.ProxyHandler({"http": PROXY_SERVER}))
            if user and pwd:
                opener.append(auth_handler(url, user, pwd))

            urllib2.install_opener(urllib2.build_opener(*opener))
            try:
                urlObj = urllib2.urlopen(request)
            except urllib2.HTTPError as e:
                if e.code in HTTP_TEMPORARY_ERROR_CODES and tries < retry:
                    time.sleep(randint(*RETRY_WAIT_TIME_RANGE))
                    tries += 1
                    continue
                else:
                    raise e

            # check whether the data stream is compressed
            if urlObj.headers.get('Content-Encoding') == 'gzip':
                return self._getUncompressedStream(urlObj)

        return urlObj

    @staticmethod
    def _getHTTPBasicAuthOpener(url, user, pwd):
        ''' returns an opener, capable of handling http-auth '''
        passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passman.add_password(None, url, user, pwd)
        auth_handler = urllib2.HTTPBasicAuthHandler(passman)
        return auth_handler

    @staticmethod
    def _getHTTPDigestAuthOpener(url, user, pwd):
        '''
        returns an opener, capable of handling http-digest authentification
        '''
        passwdmngr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passwdmngr.add_password('realm', url, user, pwd)
        auth_handler = urllib2.HTTPDigestAuthHandler(passwdmngr)
        return auth_handler

    @staticmethod
    def _getUncompressedStream(urlObj):
        ''' transparently uncompressed the given data stream
            @param[in] urlObj
            @returns an urlObj containing the uncompressed data
        '''
        compressedStream = io.BytesIO(urlObj.read())
        return GzipFile(fileobj=compressedStream)

    def _throttle(self):
        ''' delays web access according to the content provider's policy '''

        if (time.time() - self.last_access_time) < \
                DEFAULT_WEB_REQUEST_SLEEP_TIME:
            time.sleep(self.sleep_time)
        self.last_access_time = time.time()

    def __enter__(self):
        ''' support of the context protocol '''
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        ''' context protocol support '''
        if exc_type is not None:
            log.critical("%s" % exc_type)

    @staticmethod
    def get_user_password(url):
        ''' returns the url, username, password if present in the url
        @param url: well formed url, starting with a schema
        @return: tuple (new_url, user, password) '''
        if not url.startswith('http'):
            url = 'http://%s' % url

        split_url = urlsplit(url)
        user = split_url.username
        password = split_url.password
        if user and password:
            new_url = (split_url.scheme,
                       split_url.netloc.replace('%s:%s@' % (user, password),
                                                ''),
                       split_url.path,
                       split_url.query,
                       split_url.fragment)
            url = urlunsplit(new_url)
        else:
            assert not user and not password, 'if set, user AND pwd required'

        return url, user, password

    @staticmethod
    def add_user_password(url, user, password):
        split_url = urlsplit(url)
        return urlunsplit((split_url.scheme,
                           '%s:%s@%s' % (user, password, split_url.netloc),
                           split_url.path,
                           split_url.query,
                           split_url.fragment))



#!/usr/bin/env python

""" import cxl files into the rdf format """
# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

from xml.dom.minidom import parseString
from collections import defaultdict

from rdflib.Graph import Graph
from rdflib import Namespace, Literal

NS_WL   = Namespace("http://www.weblyzard.com/2005/03/31/wl#")
NS_RDFS = Namespace("http://www.w3.org/2000/01/rdf-schema#")


class XCL2RDF(object):
    """ converts XCL  to RDF 

        @warning
        XCL2RDF will _ignore_ statements with empty s, o or p.
    """
    
    @staticmethod
    def _createResourceDictionary(dom):
        """ @param[in] dom ... dom tree to parse
            @returns a dictionary with all resource names """
        
        concepts =  [ (c.getAttribute('id'), c.getAttribute('label')) for c in dom.getElementsByTagName("concept") ]
        return dict(concepts)
        
    @staticmethod
    def _createPropertyDictionary(dom):
        """ @param[in] dom ... dom tree to parse
           @returns a dictionary with all property names """

        properties =  [ (c.getAttribute('id'), c.getAttribute('label')) for c in dom.getElementsByTagName("linking-phrase") ] 
        return dict( properties )

    @staticmethod
    def _getIdentifier(s):
        """ returns the identifier (= String in camel case) for
            the given string """
        res = "".join( [ w.capitalize() for w in s.split() ] )
        return res[0].lower()+res[1:] 

    @staticmethod
    def _addOntologyStatement( ontology, s, p, o ):
        """ adds the given statement to the ontology using the
            wl-syntax 
        """
        # ignore statements with empty s, p, or o.
        if not p or not s or not o:
            return
        
        # returns the resource's identifier
        rid = lambda r: NS_WL[ XCL2RDF._getIdentifier(r)] 
        
        # define labels
        ontology.add( (rid(s), NS_RDFS['label'], Literal(s)) )
        ontology.add( (rid(o), NS_RDFS['label'], Literal(o)) )
        ontology.add( (rid(p), NS_RDFS['label'], Literal(p)) )
    
        # add relation(
        ontology.add( (rid(s), rid(p), rid(o)) )
    
    @staticmethod
    def toRDF(text):
        """ performs the conversion and returns RDF triples """
        dom = parseString(text)
        resourceDictionary = XCL2RDF._createResourceDictionary(dom)
        propertyDictionary = XCL2RDF._createPropertyDictionary(dom)
        
        incomingConnections = defaultdict( list )
        outgoingConnections = defaultdict( list )
        for c in dom.getElementsByTagName("connection"):
            c1, c2 =  c.getAttribute('to-id'), c.getAttribute('from-id')
            if c1 in resourceDictionary:
                incomingConnections[c2].append(c1)
            else:
                outgoingConnections[c1].append(c2)
        
        rdfGraph = Graph()
        for p, s_list in incomingConnections.items():
            for s in s_list:
                for o in outgoingConnections[p]:
                    XCL2RDF._addOntologyStatement(rdfGraph, 
                                                  resourceDictionary[s],
                                                  propertyDictionary[p],
                                                  resourceDictionary[o])
                    
        return rdfGraph  
                
        
        
# unit tests    
        
def testXCL2RDF():
        """ tests the XCL2RDF class """
        txt = open("./test/test.cxl").read()
        rdfGraph = XCL2RDF.toRDF(txt)
        open("./test/result.rdf", "w").write( rdfGraph.serialize() )
    
def testGetIdentifier():
    """ tests the creation of identifiers """
    assert XCL2RDF._getIdentifier("is true") == "isTrue"
    assert XCL2RDF._getIdentifier("IS") == "is" 

def testEmptyPredicates():
    """ tests whether we are able to translate unnamed predicates """
    txt = open("./test/test_empty_concepts.cxl").read()
    rdfGraph = XCL2RDF.toRDF(txt)
    open("./test/result2.rdf", "w").write( rdfGraph.serialize() )




import unittest

from eWRT.ws.googlealert import EmailParser


class TestEmailParserGoogleAlert(unittest.TestCase):
    
    def setUp(self):
        with open('test1', 'r') as file_conn:
            self.test_email = file_conn.read()
    
    def test_parse_urls(self):
        extracted_data = EmailParser.parse(self.test_email)
        self.assertEquals(extracted_data['urls'], ['http://www.militaryaerospace.com/articles/2013/03/DARPA-machine-learning.html'])
    
    def test_parse_body(self):
        extracted_data = EmailParser.parse(self.test_email)
        self.assertEquals(extracted_data['email_payload'], '=== News - 1 new result for [Machine Learning] ===\n\nDARPA launches PPAML artificial intelligence program to move machine ...\nMilitary & Aerospace Electronics\nThe goal of PPAML is to advance machine learning by using probabilistic\nprogramming to increase the number of people who can build machine learning\napplications; make machine learning experts more effective; and enable new\napplications that are ...\nSee all stories on this topic:\n\nThis as-it-happens Google Alert is brought to you by Google.\n- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\nDelete this Google Alert:\n\nCreate another Google Alert:\n\nSign in to manage your alerts:')

    def test_parse_recipient(self):
        extracted_data = EmailParser.parse(self.test_email)
        self.assertEquals(extracted_data['email_recipient'], 
                          'bicycle_repair_man@gmail.com')

    def test_parse_subject(self):
        extracted_data = EmailParser.parse(self.test_email)
        self.assertEquals(extracted_data['email_subject'], 
                          'Google Alert - Machine Learning')
    
class TestEmailParserSeveralUrls(unittest.TestCase):
    
    def setUp(self):
        with open('test3', 'r') as file_conn:
            self.test_email = file_conn.read()
            
    def test_parse_urls(self):
        extracted_urls = EmailParser.parse(self.test_email)
        self.assertEquals(extracted_urls['urls'], ['http://www.coventrytelegraph.net/news/coventry-news/2013/03/26/look-willenhall-man-s-python-nearly-ate-friend-92746-33060856/', 'http://careers.hereisthecity.com/job/senior-developer--researcher--ccpython-green---field-systematic-trading-platform-384325/'])
    
    def test_parse_body(self):
        extracted_data = EmailParser.parse(self.test_email)
        self.assertEquals(extracted_data['email_payload'], '''=== News - 2 new results for [Python] ===\n\nLOOK: Willenhall man's stolen python nearly ate friend\nCoventry Telegraph\nA MAN ended up in court after buying a 10ft stolen python during a drinking\nsession. Dwayne Matthews agreed to buy the African rock snake off a\ntraveller and thought he would sell it on. But in the cold light of day\nwhen he woke up the next morning he ...\nSee all stories on this topic:\n\nSenior Developer / Researcher (C#/C++/Python) Green - Field Systematic ...\nHere Is The City\nSENIOR DEVELOPER/RESEARCHER(C#/C++/PYTHON) GREEN-FIELD SYSTEMATIC TRADING\nPLATFORM BOUTIQUE SYSTEMATIC FUND MANAGER / LIECHTENSTEIN (CANDIDATES FROM\nALL EUROPEAN LOCATIONS CONSIDERED) ...\nSee all stories on this topic:\n''')
    
if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python

"""
::package eWRT.ws.conceptnet
Access to conceptnet data structures using its REST interface

::author: Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
"""

from collections import namedtuple, Counter
from itertools import chain
from json import loads
from os.path import exists
try:
    from cPickle import dump
except ImportError:
    from pickle import dump

from eWRT.access.http import Retrieve
from eWRT.util.cache import DiskCached
from eWRT.ws.conceptnet.node import Node
from eWRT.ws.conceptnet.edge import Edge

CONCEPTNET_BASE_URL = 'http://api.conceptnet.io'
CLEANUP_TRANSLATION_MAP = {'!': None, '.': None, '?': None,
                           '"': None, "'": None}

Concept = namedtuple("Concept", "language concept_name pos sense")


def tokenize(sentence): return sentence.translate(
    CLEANUP_TRANSLATION_MAP).split(" ")


@DiskCached(".conceptnet-query-cache")
def retrieve_conceptnet_query_result(query):
    ''' ::param url: the url to retrieve
        ::return: the json response to the given conceptnet query
    '''
    with Retrieve(__name__) as r:
        c = r.open(query, retry=3)
        return c.read()


class Result(object):
    ''' The result of a ConceptNet query '''

    RELEVANT_VSM_ATTRIBUTES = ('endLemmas', 'startLemmas', 'text', )

    def __init__(self, json_string, min_weight=0.0):
        '''
        ::param json_string: the conceptnet json string
        ::param min_weight: minimum confidence score required for
                           an edge to be included.
        '''
        self.edges = [Edge(edge_dict) for edge_dict in loads(json_string)['edges']
                      if edge_dict['weight'] >= min_weight]

        #edge_types = load(open("known-edge-types.awi")) if exists("known-edge-types.awi") else set()
        #edge_types.update([ e['rel'] for e in self.edges ])
        #dump(edge_types, open("known-edge-types.awi","w"))

    def apply_edge_filter(self, filter_list):
        '''
        Applies the given filter to the result object
        ::param filter_list: a list containing keys and
             potential values for these keys. The entry is filtered
             if it does not match any of the given key value pairs.
             (e.g. [('rel', '/r/isA'), ('rel', '/r/HasContext'), ...])
        '''
        self.edges = [edge for edge in self.edges if
                      [True for key, value in filter_list
                       if edge[key] == value]]

    def apply_language_filter(self, valid_languages):
        '''
        Removes all edges where either the start or end node
        does not matches the list of valid languages
        ::param valid_languages: list of valid languages
        '''
        self.edges = [edge for edge in self.edges if
                      Node.lang(edge['start']) in valid_languages and
                      Node.lang(edge['end']) in valid_languages]

    def get_concept(self, filter_url=None, include_subconcepts=False):
        ''' ::param filter_url: an optional filter_url for the concepts to extract
            ::param include_subconcepts: whether to include subconcepts, if filtering
                       is enabled.
                       (e.g. '/c/en/battery/n/the_battery_used_to_heat_the_filaments_of_a_vacuum_tube'
                             for '/c/en/battery')
        '''
        def matches_filter(url): return url.startswith(filter_url) if \
            include_subconcepts else lambda url: url == filter_url

        return [Node(url, self.edges) for url in chain(
                [e['start'] for e in self.edges],
                [e['end'] for e in self.edges])
                if not filter_url or matches_filter(url)]

    def get_senses(self):
        ''' ::return: the set of senses present in the result '''
        return set(self.get_concept())

    def get_vsm(self, stopword_list):
        '''
        ::param stopword_list: an optional stopword_list to apply
        ::return: a counter object with all words and their respective
                  counts for usage in a vector space model.
        '''
        vsm = Counter()

        for attr_value in (edge[attr] for edge in self.edges for attr
                           in self.RELEVANT_VSM_ATTRIBUTES
                           if attr in edge):

            # handle list values
            if isinstance(attr_value, list):
                attr_value = chain(*map(tokenize, attr_value))
                vsm.update(attr_value)
            else:
                vsm.update(tokenize(attr_value))

        # apply stopword list
        map(vsm.pop, [s for s in stopword_list if s in vsm])
        return vsm



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
EmailParser
===========
EmailParser supplies the following methods:
* EmailParser.parse(): extract the recipient, the subject, the payload and all
  valid urls from an email string.
  
*Note*: Please always follow PEP8.

@author: Philipp Konrad
'''
import email
import unittest
import re
import doctest


class EmailParser(object):
    '''Please refer to the classmethod `EmailParser.parse()`.'''

    @classmethod
    def parse(cls, string_message):
        '''
        EmailParser.parse()
        ===================
        The classmethod takes a string representing an email and returns
        all containing urls in a list::

        >>> email_string = open('test_googlealert/test1', 'r').read()
        >>> parsed_data = EmailParser.parse(email_string)
        >>> parsed_data['urls']
        ['http://www.militaryaerospace.com/articles/2013/03/DARPA-machine-learning.html']

        Another possibility is to parse a whole email and it's content.
        The library returns a dictionary with the fields `urls` and 
        `email_body`::

        >>> email_string = open('test_googlealert/test2', 'r').read()
        >>> parsed_data = EmailParser.parse(email_string)
        >>> parsed_data['urls']
        ['http://www.wu.ac.at', 'http://www.tuwien.ac.at/en/']
        >>> parsed_data['email_payload']
        'This is an email body.'

        Finally, the classmethod's returned dictionary contains an 
        `email_subject` key and an `email_recipient`::

        >>> email_string = open('test_googlealert/test3', 'r').read()
        >>> parsed_data = EmailParser.parse(email_string)
        >>> parsed_data['email_subject']
        'Very important news!'

        >>> parsed_data['email_recipient']
        'bicycle_repair_man@pythonmail.com'
        '''
        parsed_message = email.message_from_string(string_message)
        returned_data = dict()
        valid_urls = None

        if cls._is_email(parsed_message):
            email_payload = parsed_message.get_payload()
            valid_urls = cls._find_valid_urls(email_payload)

            returned_data['email_subject'] = parsed_message['subject']
            returned_data['email_recipient'] = parsed_message['To']
            returned_data['urls'] = valid_urls
            returned_data['email_payload'] = cls._clean_payload(email_payload,
                                                                valid_urls)
        else:
            raise TypeError('The supplied string is not a valid email.')

        assert len(returned_data) == 4
        return returned_data

    @classmethod
    def _is_email(cls, possible_email_obj):
        try:
            if '@' in possible_email_obj['To']:
                is_email = True
        except TypeError, e:
            is_email = False

        return is_email

    @classmethod
    def _url_is_from_google(cls, url):
        return url.startswith('http://news.google') or \
            url.startswith('http://www.google')

    @classmethod
    def _find_valid_urls(cls, email_payload):
        found_urls = re.findall('(http://.*)', email_payload)

        retval = []
        for url in found_urls:
            if not cls._url_is_from_google(url):
                if url.endswith('>'):
                    retval.append(url.rstrip('>'))
                else:
                    retval.append(url)

        return retval

    @classmethod
    def _clean_payload(cls, email_payload, urls_to_clean):

        returned_string = []

        # iterate over email payload and only add lines without url to
        # returned body.
        for line in email_payload.splitlines():

            for url in urls_to_clean:
                if url in line or 'http://' in line:
                    break
            else:
                returned_string.append(line)

        return '\n'.join(returned_string)


if __name__ == '__main__':
    doctest.testmod()



#!/usr/bin/env python

# (C)opyrights 2008-2009 by Heinz-Peter Lang <heinz@langatium.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import print_function

__version__ = "$Header$"

import logging
import unittest
from eWRT.util.exception import SNMPException


def sendSNMPTrap(message, module, level):
    '''
    sends a SNMP message
    @param message: String that should be sent
    @param level: SNMP levels ('ok', 'warning', 'critical', 'unknown') 
    '''
    try:
        raise SNMPException(module, message, level=level)
    except SNMPException:
        print('Sent a SNMP "%s": "%s"' % (level, message))


class SNMPHandler(logging.Handler):
    ''' Logging handler for sending SNMP traps'''

    def __init__(self, moduleName):
        ''' constructor calls the logging handler'''
        logging.Handler.__init__(self)
        self.moduleName = moduleName

    def emit(self, record):
        ''' sends the message '''
        level = record.levelname
        if level == 'CRITICAL' or level == 'ERROR':
            level = 'critical'
        elif level == 'WARNING':
            level = 'warning'
        elif level == 'DEBUG' or level == 'INFO':
            level = 'ok'
        else:
            level = 'unknown'

        sendSNMPTrap(record.msg, self.moduleName, level)



#!/usr/bin/env python

'''
::package eWRT.input.csv
Provides support for reading and filtering data from
CSV files based on eval expressions.

::author: Albert Weichselbraun <albert@weichselbraun.net>
'''

from csv import DictReader

# these methods will be available for third party
# modules
from numpy import var, mean


def get_csv_data(csv_file, column_expression_list, filter_expression):
    '''
    ::param csv_file: the file handle of the csv file to process
    ::param column_expression: an eval expression to extract
                               the column data.
                               e.g. ("row['correct']", "row['sv']", )
                                    "max(row['sv'], row['sentiment'])"
    ::filter_expression: a filter expressions that get's
                   evaluate using eval.
                   e.g. "row['correct'] == 'True' and
                         row['sv'] > 0."
    '''
    return [[eval(column_expression) for column_expression
             in column_expression_list]
            for row in DictReader(csv_file) if eval(filter_expression)]



#!/usr/bin/env python
"""
 @package eWRT.input.clean.text
 cleans up text phrases
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
# The code published in this module is either under the GNU General
# Public License (see below) or under the license specified in the
# function.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from eWRT.stat.string.spelling import SpellSuggestion
import re


class PhraseCleanup(object):
    """
    @class PhraseCleanup
    takes an input phrase and returns a list of "elementary" phrases.
    (e.g. quick/speedy output -> ["quick output", "speedy output"])
    """

    def __init__(self, strCleanupModules, phraseCleanupModules, wordCleanupModules):
        """ @param[in] the list of cleanupModules to apply to the input """
        self.strCleanupModules = strCleanupModules
        self.phraseCleanupModules = phraseCleanupModules
        self.wordCleanupModules = wordCleanupModules

    @staticmethod
    def getFullCleanupProfile():
        """ returns the full cleanup profile using all cleanup modules """
        strCleanupPipe = (unicode.lower, RemovePossessive(), FixDashSpace())
        phrCleanupPipe = (SplitEnumerations(),
                          SplitMultiTerms(), SplitBracketExplanations())
        wrdCleanupPipe = (FixSpelling(), RemovePunctationAndBrackets(),)
        return PhraseCleanup(strCleanupPipe, phrCleanupPipe, wrdCleanupPipe)

    def clean(self, phrase):
        """ @param[in] the input phrase to clean 
            @returns a list of elementary cleaned phrases """

        for m in self.strCleanupModules:
            phrase = m(phrase)

        phrase = [phrase]
        for m in self.phraseCleanupModules:
            phrase = m(phrase)

        result = []
        for atomicPhrase in phrase:
            outWords = atomicPhrase.split()
            for m in self.wordCleanupModules:
                outWords = m(outWords)

            result.append(" ".join(outWords))

        return result


class CleanupPipeEntry(object):
    """ @interface CleanupPipeEntry
        an entry of the cleanup pipe """


class StringCleanupModule(CleanupPipeEntry):
    """ @interface StringCleanupModule
        abstract class for all string based cleanup modules
    """

    def __call__(self, s):
        """ cleans the following list of words
            @param[in] s the string to clean
        """
        raise NotImplemented


class RemovePossessive(StringCleanupModule):
    """ @class RemovePossessive
        removes the possessive as indicated by 's """

    def __call__(self, s):
        return s.replace("'s", "")


class FixDashSpace(StringCleanupModule):
    """ @class FixDashSpace
        fixes spaces before/after dashes
        e.g. "semi- automatically" -> "semi-automatically"
             "semi -quantitative" -> "semi-quantitative"
    """
    RE_DASH = re.compile("(\w)(?:\s-\s?|-\s)(\w)")

    def __call__(self, s):
        return FixDashSpace.RE_DASH.sub(r"\1-\2", s)


class PhraseCleanupModule(CleanupPipeEntry):
    """ @interface PhraseCleanupModule
    """

    def __call__(self, l):
        """ @param[in] l a list of phrases to cleanup """
        raise NotImplemented


class SplitEnumerations(PhraseCleanupModule):
    """ @class SplitEnumerations
        splits enumerations such as
         a) first, b) second, ...  -> ['first', 'second', ...  ]
    """
    RE_ENUM = re.compile("\(?[1-9a-h*][).] ")

    def __call__(self, l):
        result = []
        for p in l:
            result.extend([s.replace(",", "").strip()
                           for s in SplitEnumerations.RE_ENUM.split(p) if s])

        return result


class SplitBracketExplanations(PhraseCleanupModule):
    """ @class SplitBracketExplanations
        removes additional information/explanations provided in brackets
        fire alert procedure (fap) -> ['fire alert procedure', 'fae' ]
    """
    RE_BRACKET = re.compile("(.+?)\(([^)]{2,})\)")

    def __call__(self, l):
        result = []
        for p in l:
            result.extend(
                [s.strip() for s in SplitBracketExplanations.RE_BRACKET.split(p) if s])

        return result


class SplitMultiTerms(PhraseCleanupModule):
    """ @class SplitMultiTerms
        splits multiple meanings into single phrases """

    RE_CHOICES = re.compile("(.*?)(\w{2,}[^-])\s*/\s*(\w{2,})\s*(.*)")

    def __call__(self, l):
        result = []
        for p in l:
            for pp in p.split(", "):
                if "/" in p:
                    m = SplitMultiTerms.RE_CHOICES.search(pp)
                    if m:
                        result.append("%s%s %s" %
                                      (m.group(1), m.group(2), m.group(4)))
                        result.append("%s%s %s" %
                                      (m.group(1), m.group(3), m.group(4)))
                        continue

                result.append(pp)
        return result


class WordCleanupModule(CleanupPipeEntry):
    """ @cleanup WordCleanupModule
    """

    def __call__(self, l):
        """ cleans the following list of words
            @param[in] l the list of words to clean
        """
        raise NotImplemented


class FixSpelling(WordCleanupModule):
    """ @class FixSpelling 
        fixes spelling mistakes """

    def __init__(self, s=None):
        """ @param[in] s optional SpellSuggestion object to use
                         for the spell checking 
        """
        WordCleanupModule.__init__(self)

        if s == None:
            self.s = SpellSuggestion()
        else:
            self.s = s

    def __call__(self, l):
        return [self.s.correct(w)[1] for w in l]

    def numMistakesFixed(self, l):
        """ @returns the number of mistakes fixed by the
                     spelling module """
        return len([True for w in l if self.s.correct(w)[1] != w])


class RemovePunctationAndBrackets(WordCleanupModule):
    """ @class RemovePunctationAndBrackets
        this should be the last module to call, as it removes too much for
        many other modules to work correctly
    """

    def __call__(self, l):
        return [s.replace(")", "").replace("(", "").replace(".", "").
                replace("'", "").replace("!", "") for s in l]


class TestPhraseCleanup(object):

    def __init__(self):
        self.p = PhraseCleanup.getFullCleanupProfile()

    def testAtomicPhrases(self):
        """ verifies that input phrases which contain multiple
            meanings get split"""
        assert self.p.clean(
            u"quick/speedy output") == [u"quick output", u"speedy output"]
        assert self.p.clean(u"i/o error") == [u"i/o error", ]
        assert self.p.clean(u"planning/design") == [u"planning", u"design"]
        print self.p.clean(u'defective product/ services')
        assert self.p.clean(
            u'defective product/ services') == [u'defective product', u'defective services']

    def testFixSpellingErrors(self):
        print self.p.clean(u"determine mening")
        assert self.p.clean(u"deterrmin mening") == [u"determine meaning", ]

    def testFixDashSpace(self):
        assert self.p.clean(
            u"semi -automatically and semi- quick") == [u"semi-automatically and semi-quick"]
        assert self.p.clean(u"run-/config") == [u"run-/config"]

    def testRemoveEnumerations(self):
        print self.p.clean(u"1. fix it, 2. do it")
        assert self.p.clean(u"1. fix it, 2. do it") == [u"fix it", u"do it"]
        assert self.p.clean(u"1) fix it, 2) do it") == [u"fix it", u"do it"]
        assert self.p.clean(u"(1) fix it, (2) do it") == [u"fix it", u"do it"]
        assert self.p.clean(u"(*) fix it, (*) do it") == [u"fix it", u"do it"]
        assert self.p.clean(u"a) fix it, b) do it") == [u"fix it", u"do it"]

        # mistakes found in applications
        print self.p.clean(u"1. life cycle phase 2. risk management tasks 3. risk management activities")
        assert self.p.clean(u"1. life cycle phase 2. risk management tasks 3. risk management activities") == \
            [u"life cycle phase", u"risk management tasks",
                u"risk management activities"]

    def testSplitBracketExplanations(self):
        # mistakes found in applications
        # cha -> ha; dow -> now due to the non risk-specific spell checking!
        assert self.p.clean(u'concept hazard analysis(cha)') == [
            u'concept hazard analysis', u'ha']
        print self.p.clean(u'dow fire and explosion index (f&ei)')
        assert self.p.clean(u'dow fire and explosion index (f&ei)') == [
            u'now fire and explosion index', u'f&ei']

    def testNumMistakesFixed(self):
        """ @test 
            returns the number of mistakes fixed in the given phrase 
        """
        s = FixSpelling()
        print s.numMistakesFixed(u'dow fire and explossion index (f&ei)'.split(" "))
        assert s.numMistakesFixed(
            u'dow fire and explossion index (f&ei)'.split(" ")) == 2



'''
Exceptions for more fine-grained errors that can occurr in web service clients.
'''


class AuthenticationError(Exception):
    '''
    Raise this exception if a request fails due to authentication issues.
    '''
    pass



#!/usr/bin/env python

''' @package eWRT.util.async
    asynchronous procedure calls

    @warning
    this library is still a draft and might change considerable

'''
from __future__ import print_function


# (C)opyrights 2008-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__copyright__ = "GPL"

import time
import os
import gzip

from shutil import rmtree
from os.path import join, exists
from cPickle import load, UnpicklingError
from subprocess import Popen
from glob import glob

try:
    import hashlib
    HASH = hashlib.sha1
except ImportError:
    import sha
    HASH = sha.sha

from eWRT.util.cache import DiskCache


class Async(DiskCache):
    ''' Asynchronous Call Handling '''

    def __init__(self, cache_dir, cache_nesting_level=0, cache_file_suffix="", max_processes=8, debug_dir=None):
        ''' initializes the Cache object
            @param[in] cache_dir the cache base directory
            @param[in] cache_nesting_level optional number of nesting level (0)
            @param[in] cache_file_suffix optional suffix for cache files
            @param[in] max_processes maximum number of parallel processes
            @param[in] optional debug directory where stdout and stderr of the processes gets saved
        '''

        self.cache_dir = cache_dir
        self.cache_file_suffix = cache_file_suffix
        self.cache_nesting_level = cache_nesting_level
        self.max_processes = max_processes
        self.cur_processes = []
        self.debug_dir = debug_dir

    def getPostHashfile(self, cmd):
        ''' returns an identifier representing the object which is compatible
            to the identifiers returned by the eWRT.util.cache.* classes. '''
        args = (tuple(
            cmd[1:]), ())  # required to produce the same hash as DiskCache's fetch method
        return self._get_fname(DiskCache.getObjectId(args))

    def post(self, cmd):
        ''' checks whether the given command is already cached and calls
            the command otherwise.
            @param[in] cmdline command to call
            @returns the hash required to fetch this object
        '''
        cache_file = self.getPostHashfile(cmd)
        # print "I will return %s for %s." % (cache_file, " ".join(cmd) )

        # try to fetch the object from the cache
        if exists(cache_file):
            try:
                load(open(cache_file))
                return cache_file
            except (EOFError, UnpicklingError):
                pass

        self._execute(cmd)
        return cache_file

    def has_processes_limit_reached(self):
        ''' closes finished processes and verifies whether we have already
            reached the maximum number of processes '''
        # verify whether all registered processes are still running
        for pObj in self.cur_processes:
            if pObj.poll() != None:
                self.cur_processes.remove(pObj)

        pids = [str(pObj.pid) for pObj in self.cur_processes]
        return len(self.cur_processes) >= self.max_processes

    def _execute(self, cmd):
        while self.has_processes_limit_reached():
            time.sleep(5)

        if self.debug_dir:
            fname_base = join(self.debug_dir, str(time.time()))
            stdout = open(fname_base + ".out", "w")
            stderr = open(fname_base + ".err", "w")
            pObj = Popen(cmd, stdout=stdout, stderr=stderr)
            os.rename(fname_base + ".out",
                      join(self.debug_dir, "debug_%d.out" % pObj.pid))
            os.rename(fname_base + ".err",
                      join(self.debug_dir, "debug_%d.err" % pObj.pid))
        else:
            pObj = Popen(cmd)

        # set stdout and stderr to non blocking because otherwise
        # the process will block after a limit of 64k has been reached
        # (see http://bytes.com/topic/python/answers/741393-spawning-process-subprocess)
        # fcntl(stdout, F_SETFL, fcntl(stdout, F_GETFL) | os.O_NONBLOCK)
        # fcntl(stdin, F_SETFL, fcntl(stdin, F_GETFL) | os.O_NONBLOCK)

        self.cur_processes.append(pObj)

    def fetch(self, cache_file):
        self.has_processes_limit_reached()
        while True:
            if exists(cache_file):
                try:
                    return load(gzip.open(cache_file))
                except (EOFError, UnpicklingError) as e:
                    print("Error opening %s" % cache_file, e)
                    pass

            time.sleep(10)

# $Id$



#!/usr/bin/env python
# encoding: utf-8
'''
 @package eWRT.ws.stat.string.normalize
 Normalizes strings prior to the application of distance metrics
'''

# (C)opyrights 2015 by Albert Weichselbraun <albert@weichselbraun.net>
#
# The code published in this module is either under the GNU General
# Public License (see below) or under the license specified in the
# function.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# Sources for look-alike characters:
# - http://ux.stackexchange.com/questions/53341/are-there-any-letters-numbers-that-should-be-avoided-in-an-id
# - pwgen source code
OCR_TRANSLATION_TABLE = {ord(u'8'): u'B',
                         ord(u'6'): u'G',
                         ord(u'1'): u'I',
                         ord(u'l'): u'I',
                         ord(u'0'): u'O',
                         ord(u'Q'): u'O',
                         ord(u'5'): u'S',
                         ord(u'2'): u'Z',
                         ord(u'Ä'): u'A',
                         ord(u'Ö'): u'O',
                         ord(u'Ü'): u'U',
                         ord(u'ß'): u'B',
                         ord(u'ä'): u'a',
                         ord(u'ö'): u'o',
                         ord(u'ü'): u'u',
                         ord(u'é'): u'e',
                         ord(u'û'): u'u',
                         ord(u'à'): u'a',
                         ord(u'â'): u'a',
                         }


def ocr_normalize(ocr_text):
    '''
    normalizes OCR text - i.e. provides a normalized representation of 
    characters that are frequently confused.

    ::param ocr_text:
        the unicode representation of the text to normalize
    ::returns:
        the normalized text
    '''
    return ocr_text.translate(OCR_TRANSLATION_TABLE)


# -------------------------------------------------------------------------
#
# Unittests
#
# -------------------------------------------------------------------------

def test_ocr_normalize():
    assert ocr_normalize(u'002 in Österreich') == u'OOZ in Osterreich'
    assert ocr_normalize(u'6000 Zürich') == u'GOOO Zurich'



#!/usr/bin/env python
# -*- coding: utf-8 -*-


""" @package eWRT.util.module_path
    utils for manipulating module paths

    Examples: see unittests
"""

# (C)opyrights 2013 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__revision__ = "$Id$"
__copyright__ = "GPL"

from six import string_types
from os.path import dirname, join


def get_resource(module_path, relative_path_list):
    '''
    Returns the path of the given resource relative to the module's directory.

    ::param module_path: path to the given module (obtained from __file__)
    ::param relative_path_list: a string or a list of directories as used for os.path.join
    '''
    if isinstance(relative_path_list, string_types):
        relative_path_list = (relative_path_list, )
    return join(dirname(module_path), *relative_path_list)


def test_get_resource():
    ''' verifies that get_resource yields the correct resource path '''
    path = get_resource(__file__, ('resources', 'test.xml'))
    assert join(dirname(__file__), 'resources', 'test.xml') == path


def test_get_resource_single_string():
    ''' verifies that get_resource yields the correct resource path for a
        single string argument '''
    path = get_resource(__file__, 'resources/test.xml')
    assert join(dirname(__file__), 'resources', 'test.xml') == path



#!/usr/bin/env python
# coding: UTF-8
"""
@package eWRT.ws.facebook
Access to the Facebook API.
"""
import time
import logging
import json
import sys
import unittest
import urllib

from urllib2 import HTTPError, URLError

from eWRT.lib import Webservice, Result
from eWRT.lib.ResultSet import ResultSet
from eWRT.access.http import Retrieve
from eWRT.config import FACEBOOK_ACCESS_KEY

logging.getLogger(__name__)


class FacebookWS(object):
    """ 
    @class FacebookWS
    class for fetching and storing the data of a user
    requires that the facebook API key and the facebook secret key are
    set in the configuration file. These can be retrieved from facebook
    """
    FB_OBJECT_TYPES = ['post', 'user', 'page', 'event', 'group', 'path']

    # Expires July 8, 2017
    # 9https://developers.facebook.com/docs/apps/changelog)
    GRAPH_API_VERSION = 'v2.3'

    retrieve = Retrieve('facebookWS')
    # added: class properties for storing searchTerm and searchType

    def __init__(self, term=None, objectType='all', since=None, limit=None):
        """ init """
        self.term = term

        if objectType == 'all':
            objectType = 'post'

        self.objectType = objectType

        if since and not isinstance(since, int):
            since = time.mktime(since.timetuple())

        self.since = since
        self.limit = limit

    @classmethod
    def search(cls, term, objectType="all"):
        '''
        searches for the given term 
        @param term: term to search
        @param objectType: objectType to search in (post, user, page, event, group)
        @return: search result
        '''
        args = {}
        result = []

        args['q'] = term

        if objectType in cls.FB_OBJECT_TYPES:
            args['type'] = objectType
            result = cls.makeRequest('search', args)
        elif objectType == 'all':
            # search all object types
            for obj_type in cls.FB_OBJECT_TYPES:
                args['type'] = obj_type
                result.extend(cls.makeRequest('search', args))
        else:
            raise ValueError, 'Illegal Object type %s' % (objectType)

        return result

    @classmethod
    def makeRequest(cls, path, args={}, maxDoc=None, method='get'):
        '''
        makes a request to the graph API
        @param path: path to query, e.g. feed of user/group/page 122222: 122222/feed
        @return: fetched data
        '''

        if not 'access_token' in args:
            # args['access_token'] = "b07f413baf9650d2363f1c8813ece6da" #very
            # unflexible, its hardcoded...
            args['access_token'] = FACEBOOK_ACCESS_KEY

        if method == 'post':
            args['method'] = 'POST'

        url = "https://graph.facebook.com/%s?%s" % (
            path, urllib.urlencode(args))
        result = cls._requestURL(url, maxDoc)
        return result

    def getJsonListStructure(self):
        request = None
        args = {}
        args['q'] = self.term

        if isinstance(args['q'], unicode):
            args['q'] = args['q'].encode('utf-8')

        if self.since:
            args['since'] = int(self.since)

        if self.limit:
            args['limit'] = self.limit

        if self.objectType == 'path':
            args_string = ''

            if 'q' in args:
                del args['q']

            if len(args):
                args_string = '?%s' % urllib.urlencode(args)

            request = {'method': "GET",
                       "relative_url": '%s/%s%s' % (self.GRAPH_API_VERSION,
                                                    self.term, args_string)}

        elif self.objectType in self.FB_OBJECT_TYPES:
            args['type'] = self.objectType
            request = {'method': "GET",
                       "relative_url": "/search?%s" % urllib.urlencode(args)}

        elif self.objectType == 'all':
            for obj_type in self.FB_OBJECT_TYPES:
                if obj_type == 'path':
                    continue
                args['type'] = obj_type
                request = {'method': "GET",
                           "relative_url": "/search?" + urllib.urlencode(args)}

        return request

    @classmethod
    def _requestURL(cls, url, maxDoc=None, result=None, tried=None):
        '''
        fetches the data for the give URL from the graph API
        @param url: valid graph-api-url
        @return: fetched data 
        '''

        if not result:
            result = []
        if not maxDoc:
            maxDoc = 1000
        if not tried:
            tried = False

        try:

            f = cls.retrieve.open(url)
            fetched = json.loads(f.read())
            tried = True
            logging.debug('processing url %s' % url)

            if isinstance(fetched, dict):

                if 'data' in fetched:
                    if not len(fetched['data']):
                        return result

                    result.extend(fetched['data'])

                    # process paging
                    if len(result) < maxDoc:
                        if 'paging' in fetched and 'previous' in fetched['paging']:
                            result = (cls._requestURL(fetched['paging']['previous'],
                                                      maxDoc, result))
                            print 'After processing paging', len(result)

                else:
                    # profiles for example don't contain a data dictionary
                    result.append(fetched)
                    print 'After appending fetched', len(result)

        except HTTPError, e:
            print 'Error: Bad Request for url %s: %s' % (url, e)
            if not tried:
                result = cls._requestURL(url, maxDoc, result, True)
        except URLError, e:
            print 'URLError for url %s: %s' % (url, e)
            if not tried:
                result = cls._requestURL(url, maxDoc, result, True)

        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Created on 19.11.2014

.. codeauthor:: Heinz-Peter Lang <lang@weblyzard.com>

Client for the Instagram API, see `Instagram Developer Documentation <http://instagram.com/developer/>`_
for details on how to obtain the `access_token` and usage of the API

.. todo:: use superclass Webservice and adapt this class. 

'''
import json
import requests
import logging

from datetime import datetime, timedelta

API_URL = 'https://api.instagram.com/v1/'
DEFAULT_MAX_AGE = 7  # days
DEFAULT_MAX_RESULTS = 1000


def convert_date(timestamp_str):
    ''' converts timestamp str to a datetime object'''
    if timestamp_str:
        return datetime.fromtimestamp(float(timestamp_str))


class InstagramClient(object):

    logger = logging.getLogger('eWRT.instagram')

    def __init__(self, access_token, api_url=API_URL,
                 max_age=DEFAULT_MAX_AGE,
                 max_results=DEFAULT_MAX_RESULTS):
        now = datetime.now()
        self.access_token = access_token
        self.api_url = api_url
        self.max_age = max_age if max_age else DEFAULT_MAX_AGE
        self.max_results = max_results if max_results else DEFAULT_MAX_RESULTS
        self.oldest_doc = now - timedelta(days=self.max_age)

    def _get_data(self, url, params=None):
        self.logger.debug('making request to %s' % url)

        response = requests.get(url, params=params)

        return json.loads(response.text)

    def _request(self, path, params=None):
        ''' executes the request and returns a generator of the found entries
        :param path: relative path of the instagram API endpoint
        :param params: additional parameters for the API request, see the API docu 
        :type params: dict or None
        :returns: generator with the entries as dictionaries
        '''
        url = '%s%s' % (self.api_url, path)

        params = params if params else {}

        if not 'access_token' in params:
            params['access_token'] = self.access_token

        done = False
        total_docs = 0
        response_docs = 0
        max_docs = params.get('max_docs', self.max_results)

        if not 'count' in params:
            params['count'] = DEFAULT_MAX_RESULTS \
                if max_docs >= DEFAULT_MAX_RESULTS else max_docs
        if 'min_timestamp' in params:
            age_threshold = params['min_timestamp']
        else:
            age_threshold = None

        while not done:
            try:
                data = self._get_data(url, params)
            except Exception as e:
                self.logger.exception('Will stop: %s' % e)
                done = True
                continue

            response_docs = 0

            if data and 'data' in data:
                for entry in data['data']:
                    if 'created_time' in entry and \
                       age_threshold and \
                       float(entry['created_time']) < age_threshold:
                        reached_limit = True
                        break
                    else:
                        add_entry = self.check_entry(entry)

                        if add_entry:
                            total_docs += 1
                            response_docs += 1
                            yield entry

                found_pagination = 'pagination' in data
                result_empty = response_docs == 0
                reached_limit = False if not max_docs else total_docs >= max_docs

                if found_pagination and not result_empty and not reached_limit:
                    if 'next_url' in data['pagination']:
                        url = data['pagination']['next_url']
                        params = None
                    else:
                        done = True
                else:
                    done = True
            else:
                self.logger.info('... done, data not in result')
                done = True

            msg = 'done = %s, got %s / %s (request/total)' % (done,
                                                              response_docs,
                                                              total_docs)

            self.logger.debug(msg)

    def search_tags(self, tag_name):
        return self._request('tags/search', {'q': tag_name})

    def get_recent_media(self, tag_name, params=None):
        return self._request(path='tags/%s/media/recent' % tag_name,
                             params=params)

    def check_entry(self, entry):
        ''' checks if the entry is ok. TODO: implement this function  '''
        return True



#!/usr/bin/env python
"""
 @package eWRT.ontology.visualize
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#from eWRT.stat.coherence import Coherence, DiceCoherence, PMICoherence
from rdflib.Graph import Graph
from rdflib import Namespace

from commands import getoutput
from itertools import chain
from operator import itemgetter
from os.path import splitext

from tempfile import NamedTemporaryFile


NS_RDFS = Namespace("http://www.w3.org/2000/01/rdf-schema#")
NS_WL   = Namespace("http://www.weblyzard.com/2005/03/31/wl#")

RELATION_PREDICATES = ('rdfs:subClassOf', 'wl:isRelatedTo', 'wl:modifierOf', 'wl:social', 'wl:deleted')

# ontology cleanup
from eWRT.input.clean.text import *

strCleanupPipe = (lambda s:s.replace(u'\xd7', " "), RemovePossessive(), FixDashSpace() )
phrCleanupPipe = (SplitEnumerations(), SplitMultiTerms(), SplitBracketExplanations() )
wrdCleanupPipe = (RemovePunctationAndBrackets(),)
phraseCleanup = PhraseCleanup(strCleanupPipe, phrCleanupPipe, wrdCleanupPipe )

class OutputQueries(object):
    """ @class OutputQueries
        static class which collects all supported sparql queries
        for the ontology visualization """

    @staticmethod
    def _webLyzardSparqlQuery():
        constraints = [ "{ ?c1 %s ?c2; ?p ?c2; rdfs:label ?s. ?c2 rdfs:label ?o. }" % p for p in RELATION_PREDICATES ]
        return "SELECT ?s ?p ?o WHERE { %s }" % " UNION ".join( constraints )

    @staticmethod
    def _labeledGraphSparqlQuery():
        return "SELECT ?s ?p ?o WHERE { ?cs ?cp ?co. ?cs rdfs:label ?s. ?co rdfs:label ?o. ?cp rdfs:label ?p. }"


class Output(object):
    """ abstract webLyzard ontology visualization superclass """
    
    def __init__(self, ontology, sparqlQuery=OutputQueries._webLyzardSparqlQuery):
        """ @param[in] ontology    ... the ontology to visualize
            @param[in] sparqlQuery ... returns the sparqlQuery required to select the node names
          
            @see OutputQueries for possible sparqlQueries
        """
        self.ontology    = ontology if isinstance( ontology, Graph ) else Graph().parse( ontology)
        self.sparqlQuery = sparqlQuery
        
    def getOntologyStatements(self):
        """ @param[in] ontology
            @returns A list of type [(s,p,o), (s,p,o)] for all statements in the ontology
        """
        q = self.sparqlQuery()
        rel = set()
        for ss, pp, oo in self.ontology.query( q, initNs=dict(rdfs=NS_RDFS, wl=NS_WL) ):
            rel = rel.union( [ (s, p, o) for s in phraseCleanup.clean(ss) for p in phraseCleanup.clean(pp) for o in phraseCleanup.clean(oo) ] )
        return rel
    
    def __str__(self):
        """ returns the visualization in the given format """
        raise NotImplementedError
    
class SubjectObjectPairOutput(Output):
    
    def __str__(self):
        return "\n".join( ["%s,%s" % (s,o) for s,_,o in self.getOntologyStatements() ] )    


class GraphvizVisualize(Output):
    """ Provides graphviz visualizations of webLyzard ontology """
    
    WL_PREDICATE_MAPPING = {u'subClassOf' : "[]",
                            u'isRelatedTo': '[style="dashed", dir="none", label="r"]',
                            u'modifierOf' :  '[style="dashed", label="m"]',
                            u'deleted'    :  '[style="dotted", arrowhead="invempty"]',
                            u'social'     :  '[style="bold", dir="none"]'
                            }
    GRAPHVIZ_HEADER = """
        rotate="90"
        size="10.8,7.4"
        center=""
        overlap=false
        splines=true
        minsep="1.5"
    """
    def __init__(self, ontology, sparqlQuery=OutputQueries._webLyzardSparqlQuery):
        self.graphTitle = ""
        Output.__init__(self, ontology, sparqlQuery)
   
    def mapPredicate(self, p):
        """ maps a predicate to a graphviz link label
            @param[in] p
            @returns the link label [...]
        """
        try:
            wl_predicate_type = p.split("#")[1]
            return GraphvizVisualize.WL_PREDICATE_MAPPING[ wl_predicate_type ]
        except IndexError:
            return u'[label="%s",dir="back"]' % p

    def getTitle(self):
        """ returns the graph's title, if a title has been set """
        if not self.graphTitle:
            return ""

        return '\ngraph [ fontsize=20, label="%s", size="6,6" ];\n ' % self.graphTitle

        
    def __str__(self):
        stmts = self.getOntologyStatements()
        conceptDefinitions  = [ '"%s" [ label="%s" ]' % (c,c) 
                               for c in set( chain( map(itemgetter(0), stmts), map(itemgetter(2), stmts) ) ) ]
        relationDefinitions = ['"%s" -> "%s" %s' % (s,o,self.mapPredicate(p)) for s,p,o in stmts ]
        
        return "digraph G{%s\n%s\n\n%s\n\n\n%s\n}" % (GraphvizVisualize.GRAPHVIZ_HEADER,
                                                      self.getTitle(),
                                                      "\n".join(conceptDefinitions), "\n".join(relationDefinitions) )

    def createImage(self, fname, imgFormat):
        """ creates an image based on the ontology 
            @param[in] fname     ... output file name 
            @param[in] imgFormat ... image format (such as eps, png) to use
        """
        dotFile = NamedTemporaryFile(suffix=".dot", delete=False)
        dotFile.write( str(self) )
        dotFile.flush()

        getoutput('fdp -T%s %s -Elen=0.1 -Eweight=3  -Gmindist=0.05 -Nmargin="0.0,0.0" -Nfontname="Helvetica" -Nheight=0.6 -Nwidth=1.1 -Nfontsize=15 -Goverlap="4:" -Goutputorder="edgesfirst" -Gsplines="true" -Gratio=0.7 -o%s.%s' % (imgFormat, dotFile.name, fname, imgFormat) )


class TestVisualizationClass(object):

    def __init__(self):
        pass

if __name__ == '__main__':
    g = GraphvizVisualize('./test/test.rdf')
    g.graphTitle="Test"
    g.createImage( "test.rdf", "png" )




#!/usr/bin/env python

"""
@package eWRT.ws.rss
@author: Albert Weichselbraun <albert@weblyzard.com>

Retrieves RSS Feeds and the corresponding Web page.
"""
import feedparser
import unittest

from datetime import datetime
from time import mktime

from eWRT.access.http import Retrieve


HTTP_FETCH_DELAY = 0

def parse(url, last_modified=None):
    """ 
    Parses the given RSS Feed an returns all articles and the content of
    the page referenced in the <link> tag.
    
    @param url: the url of the rss feed
    @param last_modified: a datetime object that specifies the last time the
                          feed has been queried the last time (only newer 
                          entries are returned).  
    """
    feed = feedparser.parse(url, modified=last_modified)
    retrieve = Retrieve("rss", HTTP_FETCH_DELAY)
    
    result = []
    for item in feed['items']:
        if datetime.fromtimestamp(
                mktime(item['updated_parsed'])) > last_modified:
            item['content'] = retrieve.open(item['link']).read()
            result.append(item)

    return result



import re
import rdflib
import unittest

from xml.dom.minidom import parse, parseString
# from metric import LearningAccuracy
# from pygments.unistring import No
from rdflib import plugin
from rdflib.store import Store
from rdflib.Graph import Graph
from rdflib import URIRef, Literal, BNode, Namespace
from rdflib import RDF
from rdflib.sparql.bison import Parse
from itertools import chain


HIERARCHY_LINK_TYPES = [URIRef('http://www.w3.org/2000/01/rdf-schema#subClassOf')]
IGNORE_HIERARCHY_LINK_TYPES = True

class OntologyEvaluator(object):
    

    def __init__(self, rdf, root_node):
        ''' constructor 
            @param rdf: rdf document to work with
            @param root_node: the root node of the RDF tree 
        '''
        self.rdf = Graph()
        self.rdf = self.rdf.parse(rdf)
        self.root_node = URIRef(root_node)
        self.nodes = {}
        self.counterAllNodes = 0 # otherwise only nodes with a specified hierarchy link type will be counted
        self._buildTree()
        
    
    def getSimilarity(self, correct_node, response_node):
        ''' find the similarity between 2 nodes 
            @param correct_node: URI of the correct node, e.g. http://semanticweb.net/c7
            @param response_node: URI of the response node, e.g. http://semanticweb.net/c7
            @return: learning accuracy (LA)
        '''
        if correct_node == response_node:
            
            return 1
        
        else:
    
            cn = self.nodes[URIRef(correct_node)]
            rn = self.nodes[URIRef(response_node)]    
    
            # sp = shortest length from root to the key concept
            # fp = shortest length from root to the predicted concept
            # dp = shortest length from msca to the predicted concept
            # cp = shortest length from root to the MSCA                
            sp = float(cn.spLen)
            fp = float(rn.spLen)

            cp, msca = OntologyEvaluator.findMSCA(cn.pathsToRoot, rn.pathsToRoot)
            self.cp = cp

            dp = float(len(rn.getShortestPathToNode(msca)))

            la = cp / ( fp + dp )
            
            bdm = self.calcBDM(cn, rn)
            
            return la, bdm

    
    def getAverageChainLength(self):
        ''' calculate the average chain length for the whole tree 
            @return: average chain length
        '''
    
        sum = 0 
        counter = 0

    
        for name, node in self.nodes.iteritems():
            sum += node.spLen
            counter +=1
            
        return float(sum) / float(counter)
    
    
    def calcBDM(self, concept, response):
        ''' calculate the balanced distance metric (BDM)
            @param concept: concept node
            @param response: response node
            @return: bdm   
        '''

        avgChainLength = float(self.getAverageChainLength())           # n0
        conceptAvgLen = float(self.averageLengthConcept(concept))      # n2
        responseAvgLen = float(self.averageLengthConcept(response))    # n3

        br = float(self.calcBR(concept))
        cp = self.cp
        dpk = concept.spLen
        dpr = response.spLen
        
        bdm = (br * (cp / avgChainLength)) / ( br * ( cp / avgChainLength ) + (dpk / conceptAvgLen ) + ( dpr / responseAvgLen ) )
    
        return bdm
    
    def calcBR(self, concept):
        ''' calculate the branching factor of the concept node
            @param conept: concept node
            @return: branching factor
        '''
        
        
        self._calculateBR()
        
        sum = 0
        counter = 0
        
        for node in self.nodes.itervalues():
            
            sum += node.br
            counter += 1
        
        br =  float(concept.br) / (float(sum) / float(counter))
        
        if br == 0:
            br = 1       
        
        return br
        
    
    def averageLengthConcept(self, node):
        
        
        sum = 0
        counter = 0 
        
        for path in node.pathsToNode[self.root_node]:
            
            sum += len(path) 
            counter +=1
        
        return float(sum) / float(counter)
    
    @staticmethod
    def findMSCA(key, response):
        ''' 
            find the most specific common abstraction (MSCA) 
            @param key: concept node
            @param reponse: response node
            @return: longest common path of both nodes  
        '''
        
        
        commonPaths = []
        
        # todo: good idea -> O(n^2)
        
        if len(key) > len(response):
            array1 = response
            array2 = key
        else:
            array1 = key
            array2 = response
            
        for path1 in array1:
        
            for path2 in array2:
                foundPath = OntologyEvaluator.compareLists(path1, path2, [])

                if len(foundPath) > 1:
                    commonPaths.append(foundPath)
                    
        return OntologyEvaluator.getLongestCommonPathLen(commonPaths)
        
    @staticmethod                  
    def compareLists(array1, array2, path=[]):
        '''
            compares two lists for their common path
            @param array1: path to root
            @param array2: path to root 
            @param path: for recursion usage of this function, stores the found path
            @return: common path (reversed: from root)
        '''

        if (len(array1) > 0 and len(array2) > 0) and (len(path) < len(array1) and len(path) < len(array2)):

            position = (len(path)) + 1
            
            element1 = array1[-position]
            element2 = array2[-position]

            if element1 == element2:
                path.append(element1)

                return OntologyEvaluator.compareLists(array1, array2, path)

        return path
    
    @staticmethod
    def getLongestCommonPathLen(paths):
        ''' 
            get the len of the most common path
            @param paths
            @return: most common path len
        ''' 
        
        if type(paths).__name__ == 'list' and len(paths) > 0:
            
            if type(paths[0]).__name__ == 'list':
    
                shortestPath = []
                
                for path in paths:
                    if len(path) < len(shortestPath) and len(path) > 0:
    
                        shortestPath = path
                
                return len(path), URIRef(path[-1])
                
            else:
    
                return len(paths), URIRef(paths[-1])
            
            
            
        else:

            return 0, None       

    def getPredicateObjects(self, subject):
        """ @param node: URI of a node, e.g. http://purl.org/dc/elements/1.1/title """
        return list(self.rdf.predicate_objects(URIRef(subject)))
    
    
    def _isRootNode(self, elementURI):
        """ evaluates if the given element URI is the root node
            @param elementURI: URI of the element
            @return: boolean 
        """
        
        if URIRef(elementURI) == self.root_node:
            return True
        else:
            return False

    def _buildTree(self):
        ''' build the RDF tree and store the parent/children relationship '''

        node = Node(self.root_node, self)
        self.nodes[self.root_node] = node

        for sub, obj, pr in self.rdf:

            if not self.nodes.has_key(sub):
                node = Node(sub, self)
                self.nodes[sub] = node

            
            if (obj in HIERARCHY_LINK_TYPES or IGNORE_HIERARCHY_LINK_TYPES) and sub != self.root_node:
                self.nodes[sub].addParent(pr)

                if not self.nodes.has_key(pr):
                    new_node = Node(pr, self)
                    self.nodes[pr] = new_node
                    
                    
                self.nodes[pr].addChild(sub)

    def _calculateBR(self, node = None):
        ''' find all descendents and calculate the branching factor
            @param node: node to start from
            @return: branching factor
        '''

        if node == None:
            node = self.nodes[self.root_node]
        
        br = 0

        for child in node.children:
            br += self._calculateBR(self.nodes[child]) + 1
            node.br = br

        return br 

# idea: create a class for the rdf-tree? and a class for the OntologyEvaluatorTools

class Node(object):
    

    def __init__(self, uri, oe):

        self.evaluator = oe
        self.uri = uri
        self.nodeURI = URIRef(uri)
        self.shortestPath = []
        self.pathsToNode = {}
        self._findPathsToNode(end=self.evaluator.root_node)
        self.br = 0
        self.parents = []
        self.children = []

    def _findPathsToNode(self, node=None, end=None, path=[]):
        ''' 
            find all paths from node to the end 
            @param node: node to start
            @param end: node to stop (default = root)
            @param path: found path
            @return: void (stored in self.pathsToNode) 
        '''

        if node == None:
            node = self.nodeURI
            path=[]
            
        if end == None:
            
            end = self.evaluator.root_node
        
        if type(node).__name__ == 'str':
            node = URIRef(node)
        
        path.append(node)
        
        if node == end:
               
            if not self.pathsToNode.has_key(end):
                self.pathsToNode[end] = []

            self.pathsToNode[end].append(path)
            return
        
        if node == self.evaluator.root_node:
            return
        
        
        for predicate, object in self.evaluator.getPredicateObjects(node):
            
            new_path = []
            new_path.extend(path)
            
            if predicate in HIERARCHY_LINK_TYPES or IGNORE_HIERARCHY_LINK_TYPES:
                self._findPathsToNode(object, end, new_path)
    
    
    def getPathsToNode(self, node):
    
        if not type(node).__name__ == 'URIRef':
            node = URIRef(node)
    
        if not self.pathsToNode.has_key(node):
            self._findPathsToNode(self.uri, node)
        
        return self.pathsToNode[node]
    
    
    def addParent(self, uri):
        ''' '''
            
        if not uri in self.parents:
            self.parents.append(uri)

    def addChild(self, uri):
        ''' '''
        
        if not uri in self.children:
            self.children.append(uri)
    
    def getShortestPathToNode(self, nodeURI):
        ''' 
            find the shortest path to the given node
            @param node: e.g. root node
            @return: list with the shortest path 
        '''

        if nodeURI == None: 
        
            return []
        
        node = URIRef(nodeURI)
        
        if not self.pathsToNode.has_key(node):
            self._findPathsToNode(self.uri, end=node, path=[])
            
        shortestPath = []    
        
        if self.pathsToNode.has_key(node):
            
            for path in self.pathsToNode[node]:
    
                if (len(path) < len(shortestPath) or len(shortestPath) == 0) and len(path) > 0:
                    shortestPath = path
                
        return shortestPath
    
    def __getShortestPathLength(self):

        if len(self.shortestPath) == 0:
            self.shortestPath = self.getShortestPathToNode(self.evaluator.root_node)
    
        return len(self.shortestPath)

        
    def __getPathToRoot(self):

        return self.getPathsToNode(self.evaluator.root_node)
        
    spLen = property(__getShortestPathLength)
    pathsToRoot = property(__getPathToRoot)

class TestOntologyEvaluator(unittest.TestCase):

    file = 'test.rdf'
    root_node = 'http://semanticweb.net/c1'
    correct_node = 'http://semanticweb.net/c4'
    response_node = 'http://semanticweb.net/c7'
    
    
    def __init__(self):
        
        self.oe = OntologyEvaluator(self.file, self.root_node)
        print self.oe.rdf.serialize(format="pretty-xml")
        
    def test_getPathToRoot(self):
        """ tests if the path was properly extracted """
    
        print '\n\n*** test finding paths to root *** \n'
    
        node = Node('http://semanticweb.net/c4', self.oe)
    
        node.getShortestPathToNode('http://semanticweb.net/c1')

        print 'found ', len(node.pathsToRoot), ' paths '
        sumLen = 0 
        for path in node.pathsToRoot:
            sumLen += len(path)
        
        assert len(node.pathsToRoot) == 2
    
        assert sumLen == 7 
    
    def test_getPathToNode(self):
        """ tests if the path was properly extracted """
    
        print '\n\n*** test finding paths to node *** \n'
    
        node = Node('http://semanticweb.net/c4', self.oe)
    
        paths = [
                [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c2'), rdflib.URIRef('http://semanticweb.net/c1')],
                [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c8'), rdflib.URIRef('http://semanticweb.net/c3'), rdflib.URIRef('http://semanticweb.net/c1')]]

        print 'paths to node = ', node.getPathsToNode('http://semanticweb.net/c3')
    
    def test_isRoot(self):
        
        assert self.oe._isRootNode(self.root_node) == True
        assert self.oe._isRootNode(self.correct_node) == False
    
    
    def test_findShortestPath(self):
        
        print '\n\n*** test finding shortest path *** \n'
        node = None
        
        node = Node('http://semanticweb.net/c4', self.oe)
        shortestPath =  node.getShortestPathToNode('http://semanticweb.net/c1')
        print 'shortestPath: ', shortestPath
        assert len(shortestPath) == 3
        assert shortestPath == [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c2'), rdflib.URIRef('http://semanticweb.net/c1')]
    
    
        node = Node('http://semanticweb.net/c4', self.oe)
        shortestPath =  node.getShortestPathToNode('http://semanticweb.net/c3')
        print 'shortestPath: ', shortestPath
        assert len(shortestPath) == 3
        assert shortestPath == [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c8'), rdflib.URIRef('http://semanticweb.net/c3')]
        
        
    def test_shortestPathLen(self):
        
        print '\n\n*** test shortest path len ***\n'
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c1')]
        print 'shortest path of c1 == ', node.spLen
        assert node.spLen == 1
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c2')]
        print 'shortest path of c2 == ', node.spLen
        assert node.spLen == 2
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c3')]
        print 'shortest path of c3 == ', node.spLen
        assert node.spLen == 2
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c4')]
        print 'shortest path of c4 == ', node.spLen
        assert node.spLen == 3
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c5')]
        print 'shortest path of c5 == ', node.spLen
        assert node.spLen == 2
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c6')]
        print 'shortest path of c6 == ', node.spLen
        assert node.spLen == 3
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c7')]
        print 'shortest path of c7 == ', node.spLen
        assert node.spLen == 3
        
        node = self.oe.nodes[URIRef('http://semanticweb.net/c8')]
        print 'shortest path of c8 == ', node.spLen
        assert node.spLen == 3 
        
        assert node.spLen == 3
        
    
    def test_compareLists(self):
        
        print '\n\n*** test comparing lists ***\n'
                
        array1 = [rdflib.URIRef('http://semanticweb.net/c7'), rdflib.URIRef('http://semanticweb.net/c2'), rdflib.URIRef('http://semanticweb.net/c1')]
        array2 = [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c2'), rdflib.URIRef('http://semanticweb.net/c1')]
        print OntologyEvaluator.compareLists(array1, array2);
        assert OntologyEvaluator.compareLists(array1, array2) == [rdflib.URIRef('http://semanticweb.net/c1'), rdflib.URIRef('http://semanticweb.net/c2')]

        print 'second test'
        path = []
        array1 = [rdflib.URIRef('http://semanticweb.net/c8'), rdflib.URIRef('http://semanticweb.net/c3'), rdflib.URIRef('http://semanticweb.net/c1')]
        array2 = [rdflib.URIRef('http://semanticweb.net/c7'), rdflib.URIRef('http://semanticweb.net/c3'), rdflib.URIRef('http://semanticweb.net/c1')]
        assert OntologyEvaluator.compareLists(array1, array2, []) == [rdflib.URIRef('http://semanticweb.net/c1'), rdflib.URIRef('http://semanticweb.net/c3')]
 
    
    def test_findMSCA(self):
        
        print '\n\n *** test finding MSCA *** \n'
        
        cnode_paths = [
            [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c2'), rdflib.URIRef('http://semanticweb.net/c1')],
            [rdflib.URIRef('http://semanticweb.net/c4'), rdflib.URIRef('http://semanticweb.net/c8'), rdflib.URIRef('http://semanticweb.net/c3'), rdflib.URIRef('http://semanticweb.net/c1')]]
        rnode_paths = [
            [rdflib.URIRef('http://semanticweb.net/c7'), rdflib.URIRef('http://semanticweb.net/c2'), rdflib.URIRef('http://semanticweb.net/c1')],
            [rdflib.URIRef('http://semanticweb.net/c7'), rdflib.URIRef('http://semanticweb.net/c3'), rdflib.URIRef('http://semanticweb.net/c1')]]
        
        len, path = OntologyEvaluator.findMSCA(cnode_paths, rnode_paths)
        
        print len, path
        assert len == 2
        assert path == rdflib.URIRef('http://semanticweb.net/c3') or path == rdflib.URIRef('http://semanticweb.net/c2')

    def test_getSimilarity(self):
        
        print '\n\n *** test getting similarity *** \n'
        self.oe = OntologyEvaluator(self.file, self.root_node)
        
        for uri, node in self.oe.nodes.iteritems():
            
            print uri, node.spLen
        
        correct_node = 'http://semanticweb.net/c4'
        respone_node = 'http://semanticweb.net/c7'
        
        print self.oe.getSimilarity(correct_node, respone_node)
        
        # todo: check if the result is correct 
    
    def test_buildTree(self):
        print '\n\n*** test build Tree ***\n'
        self.oe._buildTree()
        print self.oe.nodes
        
        for name, node in self.oe.nodes.iteritems():
            print name, node.spLen
        
        assert len(self.oe.nodes) == 8
        
    
    def test_averageChainLength(self):
        print '\n\n*** test average chain length ***\n'
        avgChainLen = self.oe.getAverageChainLength()
        print avgChainLen
        assert 2.375 == avgChainLen
        
    
    def test_averageConceptLength(self):
        
        print '\n\n*** test average concept length ***\n'
        
        node = Node('http://semanticweb.net/c4', self.oe) 
        
        avg = self.oe.averageLengthConcept(node)
        
        print avg
        
        assert 3.5 == avg
        
        
    def test_calculateBR(self):
        
        print '\n\n*** test _calculateBR ***\n'
        
        node = URIRef('http://semanticweb.net/c1')
        
        br = self.oe._calculateBR()
            
        assert self.oe.nodes[URIRef('http://semanticweb.net/c1')].br == 9
        assert self.oe.nodes[URIRef('http://semanticweb.net/c4')].br == 0
        assert self.oe.nodes[URIRef('http://semanticweb.net/c8')].br == 1
    
    def test_calcBR(self):
        
        print '\n\n*** test _calculateBR ***\n'

        assert 1.5 == self.oe.calcBR(self.oe.nodes[URIRef('http://semanticweb.net/c3')])

    def test_calcBDM(self):
        ''' '''
        print '\n\n *** test calcBDM *** \n'
        
        correct_node = URIRef('http://semanticweb.net/c4')
        response_node = URIRef('http://semanticweb.net/c7')
        
        cn = self.oe.nodes[correct_node]
        rn = self.oe.nodes[response_node]
    
        print self.oe.calcBDM(cn, rn)
        
        # todo: check if the result is correct 
    
    def test_sample_of_paper(self):
        
        print '\n\n*** testing proton ontology *** \n'
        
        # ontology can be downloaded from http://veggente.berlios.de/ns/RIMOntology
        
#        oe = OntologyEvaluator('rim-ontology.rdf', 'http://veggente.berlios.de/ns/RIMOntology#RIM_HL7')
#        oe.getSimilarity('http://veggente.berlios.de/ns/RIMOntology#Access', 'http://veggente.berlios.de/ns/RIMOntology#Role')
#        print len(oe.nodes)
#        print oe.counterAllNodes

if __name__ == '__main__':
    
    toe = TestOntologyEvaluator()
    toe.test_getPathToRoot()
    toe.test_getPathToNode()
    toe.test_isRoot()
    toe.test_findShortestPath()
    toe.test_shortestPathLen()
    toe.test_compareLists()
    toe.test_findMSCA()
    toe.test_getSimilarity()
    toe.test_buildTree()
    toe.test_averageChainLength()
    toe.test_averageConceptLength()
    toe.test_calculateBR()
    toe.test_calcBR()
    toe.test_calcBDM()
#    toe.test_sample_of_paper()


# from eWRT.config import GOOGLE_USER, GOOGLE_PASS
# 
# try:
#     from eWRTlibs.googleTrends.pyGTrends import pyGTrends
#     LOADED = True
# except ImportError:
#     from warnings import warn
#     from sys import exit
#     warn("This module requires google Trends ")
#     LOADED = False
#     
# class GoogleTrends(object):
#     ''' ## Documentation for the Class Google Trends
#         fetches the Google trends for a set of keywords '''
# 
#     
#     def __init__(self, user=GOOGLE_USER, password=GOOGLE_PASS):
#         ''' Constructor establishes the connection
#             @param user
#             @param password
#         '''
#         self.connector = pyGTrends(user, password)
#         
#  
#     def getTrends(self, keywords, date=None, section=None):
#         ''' getTrends fetches the trends
#             @param keywords
#             @param date to filter 
#             @param section
#             @return trends 
#         '''
#         if date == None:
#             self.connector.download_report(keywords)
#         else:
#             self.connector.download_report(keywords, date)
# 
#         print self.connector.header_dictionary
# 
#         if section == None:
#             trends = self.connector.csv()
#         else:
#             trends = self.connector.csv()
# 
#         return trends
# 
# if __name__ == "__main__":
#     a = GoogleTrends()
#     print a.getTrends(('Apple', 'Microsoft'))



#!/usr/bin/env python
"""
 @package eWRT.ws.stat.eval.metrics
 Standard IR evaluation metrics such as
  * precision
  * recall
  * F1 measure
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                   and others (as outlined in the functions.
#
# The code published in this module is either under the GNU General
# Public License (see below) or under the license specified in the
# function.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


def precision(relevant, retrieved):
    """ returns the precision of the given sets 
        @param[in]  relevant set of relevant terms
        @param[in] retrieved set of retrieved terms
        @returns the precision
    """
    assert isinstance(relevant, set)
    assert isinstance(retrieved, set)

    return float(len(relevant.intersection(retrieved))) / len(retrieved)


def recall(relevant, retrieved):
    """ returns the recall of the given sets 
        @param[in]  relevant set of relevant terms
        @param[in] retrieved set of retrieved terms
        @returns the recall
    """
    assert isinstance(relevant, set)
    assert isinstance(retrieved, set)

    return float(len(relevant.intersection(retrieved))) / len(relevant)


def fMeasure(p, r, beta=1.):
    """ returns the F-measure for the given precision and recall
        @param[in] p precision
        @param[in] r recall 
        @param[in] beta  weight used to compute the f mesure
        @returns the F-Measure
    """
    return (1 + beta * beta) * (p * r) / (beta * beta * p + r)


class TestEvaluationMetrics(object):
    """ tests the evaluation metrics """

    a = set((1, 3, 8, 9))
    b = set((1, 3, 10, 12))
    c = set((1, 3))
    d = set((10, 11))

    def testPrecision(self):
        assert precision(self.a, self.b) == 0.5
        assert precision(self.a, self.a) == 1.0
        assert precision(self.a, self.c) == 1.0
        assert precision(self.a, self.d) == 0.0

    def testRecall(self):
        assert recall(self.a, self.a) == 1.0
        assert recall(self.a, self.b) == 0.5
        assert recall(self.a, self.c) == 0.5
        assert recall(self.a, self.d) == 0.0

    def testFMeasure(self):
        assert fMeasure(1., 1., 1.) == 1
        assert fMeasure(1., 0., 1.) == 0.0
        assert fMeasure(1., 0.5, 1.) == 1 / 1.5



#!/usr/bin/env python
"""
 @package eWRT.ws.geonames.gazetteer.exception
 exceptions related to the gazetteer class

"""

# (C)opyrights 2009 by Heinz Lang <heinz.lang@wu.ac.at>
#                      Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class GazetteerEntryNotFound(Exception):
    """ @class GazetteerEntryNotFound
        Base class for gazetteer lookup errors 
    """

    def __init__(self, id, query):
        self.id = id
        self.query = query
        print id, query

    def __str__(self):
        return "Gazetteer lookup for entity-id '%s' failed." % (self.id)


class GazetteerNameNotFound(Exception):
    """ @class GazetteerNameNotFound
        This exception is thrown if a lookup name has not been found in the gazetteer
    """

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return "Gazetteer lookup of name '%s' failed." % (self.name)



'''
Created on Sep 02, 2014

:author: svakulenko
'''

# Bing API Version 2.0
# sample URL for web search
# https://api.datamarket.azure.com/Bing/Search/Web?$format=json&Query=%27Xbox%
# 27&$top=2

from eWRT.ws.rest import RESTClient
from eWRT.ws import AbstractIterableWebSource


class BingSearch(AbstractIterableWebSource):

    """wrapper for the Bing Search API"""

    NAME = "Bing Search"
    ROOT_URL = 'https://api.datamarket.azure.com/Bing/Search'
    DEFAULT_MAX_RESULTS = 50  # requires only 1 api access
    SUPPORTED_PARAMS = ['command', 'output_format']
    DEFAULT_COMMAND = 'Web'  # Image, News
    DEFAULT_FORMAT = 'json'
    DEFAULT_START_INDEX = 0
    RESULT_PATH = lambda x: x['d']['results']  # path to the results in json

    MAPPING = {'date': ('valid_from', 'convert_date'),
               'text': ('content', None),
               'title': 'Title',
               }

    def __init__(self, api_key, username, api_url=ROOT_URL):
        """fixes the credentials and initiates the RESTClient"""

        assert(api_key)

        self.api_key = api_key
        self.api_url = api_url
        self.username = username
        self.client = RESTClient(
            self.api_url, password=self.api_key, user=self.username,
            authentification_method='basic')

    def search_documents(self, search_terms, max_results=DEFAULT_MAX_RESULTS,
                         from_date=None, to_date=None, command=DEFAULT_COMMAND,
                         output_format=DEFAULT_FORMAT):
        """calls iterator and results' post-processor"""
        # Web search is by default
        fetched = self.invoke_iterator(search_terms, max_results, from_date,
                                       to_date, command, output_format)

        result_path = lambda x: x['d']['results']
        return self.process_output(fetched, result_path)

    def request(self, search_term, current_index,
                max_results=DEFAULT_MAX_RESULTS, from_date=None,
                to_date=None, command=DEFAULT_COMMAND,
                output_format=DEFAULT_FORMAT):
        """calls Bing Search API"""

        parameters = {'Query': search_term,
                      '$format': output_format,
                      '$top': max_results,
                      '$skip': current_index}

        # for testing purposes
        # print(current_index, max_results, search_term)

        response = self.client.execute(command, query_parameters=parameters)
        return response

    @classmethod
    def convert_item(cls, item):
        """output convertor: applies a mapping to convert
        the result to the required format
        """

        result = {'url': item['Url'],
                  'title': item['Title'],
                  }

        return result


#!/usr/bin/env python
"""
 @package eWRT.input.conv.html
 converts HTML pages into text
"""

# (C)opyrights 2009-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from eWRT.config import CMD_CONV
from eWRT.util.execute import pipe_content


class HtmlToText(object):
    """ converts HTML into text
        requires a converter
    """

    @staticmethod
    def getText(html_content, encoding="utf8"):
        """ @param[in] html_content the content of the html page to convert 
            @param[in] encoding the document encoding 
            @returns the text representation of the Web page
        """
        # check whether this is really a html file
        if not "<" in html_content or not ">" in html_content:
            return html_content

        _, html = pipe_content(CMD_CONV['html'], html_content)
        return html


class TestHtmlToText(object):

    def testConversion(self):
        text = HtmlToText.getText(
            "<html><body><h1>Hallo</h1><ul><li>1</li><li>Jasna</li></ul></body></html>")
        assert 'Jasna' in text

    def testBorderCases(self):
        assert HtmlToText.getText("") == ""
        assert HtmlToText.getText("   ") == "   "



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
@package eWRT.stat.language

language detection
'''
import re
import unittest

from glob import glob
from os.path import basename

from eWRT.util.module_path import get_resource


def read_wordlist(fname):
    ''' reads a language wordlist from a file '''
    with open(fname) as f:
        return set(map(str.lower, map(str.strip, f.readlines())))


# returns the language name based on the language file's name
def get_lang_name(fname): return basename(fname).split(".")[0]


LANG_DATA_DIR = get_resource(__file__, 'data')

##
# \var STOPWORD_DICT: a dictionary of the 100 most common words in the given language
STOPWORD_DICT = {get_lang_name(fname): read_wordlist(
    fname) for fname in glob(LANG_DATA_DIR + "/*.csv")}
DELETE_CHARS = ",.!?\"'"
DELETE_TABLE = {ch: None for ch in DELETE_CHARS}


import string
table = string.maketrans('ac', 'cx')


def detect_language(text):
    ''' detects the most probable language for a given text '''
    if not text.strip():
        return None

    text = text.lower()
    text = re.sub(r'[^\w]', ' ', text)

    cleaned_text = ''
    try:
        cleaned_text = text.translate(DELETE_TABLE)  # python3
    except Exception as e:
        cleaned_text = text.translate(None, DELETE_CHARS)  # python2.6
    words = [word.strip() for word in cleaned_text.split(" ")]
    current_lang = None
    current_wordcount = 0
    for lang, reference_wordlist in STOPWORD_DICT.items():
        wordcount = sum([1 for word in words if word in reference_wordlist])
        if wordcount >= current_wordcount and wordcount > 0:
            current_wordcount = wordcount
            current_lang = lang

    return current_lang


class DetectLanguageTest(unittest.TestCase):

    def test_detect_language(self):
        ''' tests the language detection based on examples '''

        text = u"#maryboo #health #baby #mom #food #cosmetics #love  http://t.co/nPqzL8MRKa"
        # RT @LesbianGIFs_: She's HOT AF 😩🔥 http://t.co/xZ2VFdAtNx
        print(detect_language(text))

        text = u'''Das Glück der Erde liegt in diesem Fall im Dreck. Begeistert
                  werfen sich die Schüler einer zweiten Klasse Volksschule
                  Erdklumpen auf die grünen T-Shirts. Andere rennen lachend herum,
                  reißen verstohlen Sauerampfer aus dem Beet und stecken sich die
                  Blätter in den Mund.'''

        print(detect_language(text))
        assert detect_language(text) == 'de'

        text = u'''Deux nouveaux décès par crise cardiaque en lien avec la
                  consommation de boissons énergisantes ont été signalés aux
                  autorités sanitaires, a indiqué mercredi 6 juin au soir l'agence
                  de sécurité sanitaire pour l'alimentation (Anses).'''

        assert detect_language(text) == 'fr'

        text = u'''Weniger 1V-Renten, mehr Pensionskassengelder ZÜRICH. Immer weniger Leute beziehen eine IVunsere PensionskassenGuthaben: Sie wachsen. Seit 2003 ist die Zahl der jährlich neu zugesprochenen Renten bei der Invalidenversicherung (IV) um mehr als 47% gesUnken. «Wir führen das auf die 5. IVG-Revision, die relativ stabile Wirtschaftslage und die Fortschritte bei der Wiedereingliederung zurück», sagt Zurich-Sprecher Frank Keidel. Vom starken Rückgang profitiert nicht nur die IV als erste Säule, sondern alle Erwerbstätigen. Denn dank der gesunkenen Zahl der Invaliden müssen auch die Pensionskassen weniger IV-Rentenbeiträge aus der zweiten Säule ausschütten. Unter dem Strich bleibt deshalb viel mehr Geld für die Altersvorsorge. Die Beiträge der zweiten Säule setzen sich zusammen aus IV-, Todesfall- und Kostenprämie. Eine Umfrage von 20 Minuten bei den Versicherungen zeigt, dass die Risikoprämten für Invalidität teils um 10 bis 20% gesenkt werden konnten. «Oberdurchschnittlich konnten wIr 2011 die Prämien im Gross- und Detailhandel, bei Optikern, Apotheken und Drogerien senken», so Allianz-Suisse-SprecherBernddeWall. Ebenfalls stark profitiert hätten Elektro-, Sanitär- und Lüftungsinstallateure, Maler, Gipser und Glaser Bei Swlss Life sanken die Prämien seit 2007 durchschnittlich um %. Auch Axa Wmterthur hat in den letzten Jahren die Taufe bereits dreimal gesenkt. Und mit der teilautonomen Sanimelstiftung Vita plant sie für 2013 eine durchschnittliche r? /2 1 1 IV-Renten sind rOckilufig. FOTOLIA Senkung der Prämien von rund 300 Franken pro versicherte Person. ELISABEflI RIZZI'''

        assert detect_language(text) == 'de'

    def test_exceptions(self):
        ''' results for empyt strings '''
        assert detect_language(u" ") == None
        assert detect_language(u"") == None


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" install eWRT :) """
from setuptools import setup, find_packages

from sys import exit

setup(
    ###########################################
    # Metadata
    name="eWRT",
    version="0.9.6.20180607",
    description='eWRT',
    author='Albert Weichselbraun, Heinz Lang, Gerhard Wohlgenannt, Johannes Duong, Norman Süsstrunk, Daniel Streiff',
    author_email='albert@weblyzard.com, lang@weblyzard.com, wohlg@ai.wu.ac.at, johannes.duong@wu.ac.at, norman.suestrunk@htwchur.ch, daniel.streiff@htwchur.ch',
    url='http://www.weblyzard.com/ewrt/',
    license="GPL3",
    package_dir={'': 'src'},
    install_requires=['redis',
                      'oauth2client==2.2.0',  # YT
                      'google-api-python-client==1.4.0',  # YT,
                      'six',
                      'gdata'
                      #                         'python-oauth2' #linkedin
                      ],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],


    ###########################################
    # Run unittests
    test_suite='nose.collector',

    ###########################################
    # Scripts
    scripts=['src/eWRT/input/corpus/reuters/reuters.py'],

    ###########################################
    # Package List
    packages=find_packages('src'),

    ###########################################
    # Package Data
    package_data={'eWRT.stat.language': ['data/*.csv'],
                  'eWRT.input.stock': ['data/*.csv'],
                  },
)



#!/usr/bin/env python

""" yahoo! - uses yahoo's boss search service """

# (C)opyrights 2008-2009 by Albert Weichselbraun <albert@weichselbraun.net>
#                           Heinz Peter Lang <hplang@langatium.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

from urllib import urlencode

from eWRT.access.http import Retrieve
from eWRT.config import YAHOO_APP_ID


YAHOO_TERM_EXTRACTION_URI = 'http://search.yahooapis.com/ContentAnalysisService/V1/termExtraction'

class YahooTermExtractor(object):
    """ interfaces with yahoo's search service 
        * Term extraction: extract terms from yahoo search
          http://developer.yahoo.com/search/content/V1/termExtraction.html
    """
    __slots__ = ('r', )

    def __init__(self):
        self.r = Retrieve( YahooTermExtractor.__name__ )

    def extractTerms(self, content):
        """ extract terms from yahoo search, see http://developer.yahoo.com/search/content/V1/termExtraction.html """ 

        params = urlencode( {'appid': YAHOO_APP_ID,
                             'context': content,
                             'output': 'json'
        })
        result = eval ( self.r.open(YAHOO_TERM_EXTRACTION_URI, params).read() )
        return result['ResultSet']['Result']


class TestYahooTermExtractor(object):
    """ tests the yahoo search API """

    def __init__(self):
        self.y = YahooTermExtractor()

    def testExtractTerms(self):
        """ tests the yahoo term extraction interface """
        text = ''' 
            appid       string (required)       The application ID. See Application IDs for more information.
            context     string (required)     The context to extract terms from (UTF-8 encoded).
            query     string     An optional query to help with the extraction process.
            output     string: xml (default), json, php     The format for the output. If json is requested, the results will be returned in JSON format. If php is requested, the results will be returned in Serialized PHP format.
            callback     string     The name of the callback function to wrap around the JSON data. The following characters are allowed: A-Z a-z 0-9 . [] and _. If output=json has not been requested, this parameter has no effect. More information on the callback can be found in the Yahoo! Developer Network JSON Documentation. ''' 

        keywords = self.y.extractTerms(text)
        assert 'json' in keywords
        assert 'yahoo developer network' in keywords
        assert 'the' not in keywords
        assert 'for' not in keywords





#!/usr/bin/env python

""" 
    @package DetectLanguage.reuters
    a generic method to retrieve text from the reuters corpus
"""

# -----------------------------------------------------------------------
# - (C)opyright 2009 by Albert Weichselbraun <albert@weichselbraun.net>
# -                    webLyzard technology gmbh <awe@weblyzard.com>
# -----------------------------------------------------------------------

__revision__ = "$Revision: 545 $"
__author__ = "Albert Weichselbraun"

from eWRT.config import BBC_CORPUS_LOW
from eWRT.input.conv.html import HtmlToText
from glob import glob
import os


class BBCGetCorpus(object):
    """ An iterator over all documents """

    def __init__(self, filePattern="*"):
        """ @param[in] filePattern Pattern of files to consider (e.g. 7[3456789]*.stm)
        """
        self.files = glob(os.path.join(BBC_CORPUS_LOW, filePattern))

    def __iter__(self):
        return self

    def next(self):
        if self.files:
            htmlTxt = open(self.files.pop()).read()
            return HtmlToText.getText(htmlTxt)
        else:
            raise StopIteration

    @staticmethod
    def getTitle(text):
        """ returns the title of a given text """
        return text.split("\n")[2].strip()


if __name__ == '__main__':
    def n(x): return x.replace("'", "''")
    for num, text in enumerate(BBCGetCorpus("7[3456789]*.stm")):
        title = BBCGetCorpus.getTitle(text)
        print "INSERT INTO evaluation_documents (content_id, title, content) VALUES ('%d', '%s', '%s');" % (num, n(title), n(text))



class TagInfoService(object):
    """ Class for fetching assigned tags """

    def getTagInfo(self, tags):
        """ returns the count for the given tags
            @param list/tuple of tags
            @returns number of counts """
        return NotImplemented

    def getRelatedTags(self, tags, retrieveTagInfo=False):
        """ returns a the count of related tags 
            @param   list/tuple of tags 
            @param   retrieveTagInfo  determines whether we will retrieve the tagInfo for the related tags
            @returns list of related tags with a count of their occurence """
        return NotImplemented



'''
Created on Feb 3, 2012

@author: heinz-peterlang
'''

from json import loads
from inspect import getargspec
from twisted.web import resource

from eWRT.ws.rest.server.exception import (MissingArgumentErrorMsg,
                                           UnknownArgumentErrorMsg)

class WeblyzardService(resource.Resource):
    '''
    The Weblyzard RESTless service base class
    '''

    children = {}

    # deprecated
    VALID_ARGS = None

    def __init__(self, function=None):
        '''
        @param get_function: the function to call for get requests
        '''
        argspec = getargspec(function)
        self.func_args = argspec.args
        self.func_required_args = self.func_args if not argspec.defaults \
                  else self.func_args[:-len(argspec.defaults)]
        self.func_required_args.remove('self')
        self.function = function
    
    def render_GET(self, request):
        args = { key: value[0] for key, value in request.args.items() }
        return self.call(self.function, args)
    
    def render_POST(self, request):
        args = loads( request.content.read() )
        return self.call(self.function, args)
    
    @classmethod
    def check_arguments(cls, request):
        """ @deprecated: use call instead """
        
        for arg in cls.VALID_ARGS:
            if not arg in request.args:
                return MissingArgumentErrorMsg(cls.check_arguments, arg)()
        
        return True
    
    def call(self, function, args):
        """ calls the given function if all required arguments have been
            supplied, or output its docstring otherwise.
            
            @param Function: the function to call
            @param request:  the twisted request object
            @return: the functions return value
        """
        for required_arg in self.func_required_args:
            if required_arg not in args:
                return MissingArgumentErrorMsg(function, required_arg)()
        
        #print self.func_required_args
        
        for arg in args:
            if arg not in self.func_args:
                return UnknownArgumentErrorMsg(function, arg)()
        
        return self.function(**args)



#!/usr/bin/env python
"""
 @package eWRT.ontology.eval.terminology
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from rdflib import Graph
from rdflib import Namespace

from eWRT.stat.coherence import Coherence, DiceCoherence, PMICoherence


NS_RDFS = Namespace("http://www.w3.org/2000/01/rdf-schema#")
NS_WL   = Namespace("http://www.weblyzard.com/2005/03/31/wl#")

RELATION_PREDICATES = ('rdfs:subClassOf', 'wl:isRelatedTo', 'wl:modifierOf', 'wl:social') # 'wl:deleted'

class CoherenceEvaluator(object):

    def __init__(self, metric):
        assert isinstance(metric, Coherence)
        self.metric = metric

    @staticmethod
    def _buildSparqlQuery(relation_predicates):
        constraints = [ "{ ?c1 %s ?c2; ?p ?c2; rdfs:label ?s. ?c2 rdfs:label ?o. }" % p for p in relation_predicates ]
        return "SELECT ?s ?p ?o WHERE { %s }" % " UNION ".join( constraints )

    def getWeakConcepts(self, ontology, relation_predicates=RELATION_PREDICATES):
        """ returns a list of weak concepts according to the given coherence measure 
            @param[in] ontology ... an rdflib.Graph object or an rdf file containing the concepts 
            @returns a list of "weak" concepts
        """
        rdf = ontology if isinstance( ontology, Graph ) else Graph().parse( ontology)
        q   = CoherenceEvaluator._buildSparqlQuery(relation_predicates)

        conceptEval = [ (self.metric.getTermCoherence(s,o), s, p, o) \
            for s, p, o in rdf.query( q, initNs=dict(rdfs=NS_RDFS, wl=NS_WL) ) ]

        conceptEval.sort()
        return conceptEval


class TestTermEval(object):

    TEST_ONTOLOGY = './test/test.rdf'
    
    def testSparqlQueryGeneration(self):
        """ tests the generation of Sparql queries """
        referenceQuery = "SELECT ?s ?p ?o WHERE { { ?c1 rdfs:subClassOf ?c2; ?p ?c2; rdfs:label ?s. ?c2 rdfs:label ?o. } UNION { ?c1 wl:isRelatedTo ?c2; ?p ?c2; rdfs:label ?s. ?c2 rdfs:label ?o. } UNION { ?c1 wl:modifierOf ?c2; ?p ?c2; rdfs:label ?s. ?c2 rdfs:label ?o. } }"
        print referenceQuery
        print "--"
        print CoherenceEvaluator._buildSparqlQuery( RELATION_PREDICATES )
        assert( CoherenceEvaluator._buildSparqlQuery() == referenceQuery )

    def testEvaluateTerminology(self):
        """ tests an ontology (file) and returns a list of 
            weak concepts """
        from eWRT.ws.yahoo import Yahoo
        c = DiceCoherence( dataSource = Yahoo() )
        t = CoherenceEvaluator( c )
        weakConcepts = t.getWeakConcepts( self.TEST_ONTOLOGY )
        assert isinstance( weakConcepts, list )
        print len(weakConcepts)
        assert len(weakConcepts) == 27-1
        

if __name__ == '__main__':
    from eWRT.ws.yahoo import Yahoo
    # c = PMICoherence( dataSource = Yahoo() )
    c = DiceCoherence( dataSource = Yahoo() )
    t = CoherenceEvaluator( c )
    weak = t.getWeakConcepts( "./test/test.rdf" ) 
    print weak
    print "---"
    print "\n".join( ( str(w) for w in weak ) )



#!/usr/bin/env python

""" @package eWRT.util.assert
    Assertion based counters 

    Examples: see unittests
"""

# (C)opyrights 2011 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__revision__ = "$Id$"
__copyright__ = "GPL"


from collections import Counter


class AssertReturnValue(object):
    """ decorator class used to time functions """

    def __init__(self, evalExpression, counterNameTrue, counterNameFalse):
        """
        @param evalExpression: the expression to verify (e.g. x>12); 
        @note evalExpression uses the variable x for the return value
        @param counterNameTrue: counter to increase if the condition is true
        @param counterNameFalse:counter to increase if the condition is false  
        """
        self.fn = None
        self.evalExpression = evalExpression
        self.counterNameTrue = counterNameTrue
        self.counterNameFalse = counterNameFalse
        self.counter = Counter()

    def _assertReturnValue(self, x):
        if eval(self.evalExpression):
            self.counter[self.counterNameTrue] += 1
        else:
            self.counter[self.counterNameFalse] += 1

    def __call__(self, fn):
        """
        returns the wrapper object which is called instead of the original
        function ones the decorator is used.
        """
        def wrapper(*fargs, **kw):
            returnValue = self.fn(*fargs, **kw)
            self._assertReturnValue(returnValue)
            return returnValue

        # Save wrapped function reference
        self.fn = fn
        wrapper.__name__ = fn.__name__
        wrapper.__dict__.update(fn.__dict__)
        wrapper.__doc__ = fn.__doc__
        wrapper.counter = self.counter
        return wrapper



#!/usr/bin/env python

""" @package eWRT.ws.yahoo
    support for the yahoo! search 
    
    @remarks
    this module is based on yahoo's boss search service 
"""

# (C)opyrights 2008-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                           Heinz Peter Lang <hplang@langatium.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

from urllib import urlopen, urlencode, quote
from urllib2 import URLError
from nose.plugins.attrib import attr
from socket import setdefaulttimeout

from eWRT.ws.TagInfoService import TagInfoService
from eWRT.config import YAHOO_APP_ID, YAHOO_SEARCH_URL
from eWRT.input.conv.html import HtmlToText
from eWRT.access.http import Retrieve


setdefaulttimeout(60)

class Yahoo(TagInfoService):
    """ interfaces with yahoo's search service 
        * Search: Yahoo! BOSS
          (see http://developer.yahoo.com/search/boss)
    """
    __slots__ = ('r', )

    def __init__(self):
        self.r = Retrieve( Yahoo.__name__, sleep_time=0 )

    def query(self, terms, count=0, queryParams={} ):
        """ returns search results for the given terms
            @param[in] terms       ... a list of search terms
            @param[in] count       ... number of results to return (0 if we are
                                       interested on the search meta data only).
            @param[in] queryParams ... a dictionary of query parameters to add to
                                          the request
            @returns the search results
        """
        assert ( isinstance(terms, tuple) or isinstance(terms, list) )
        queryParams.update( {'appid': YAHOO_APP_ID,
                             'count': count,
                             'format': 'json'
        } )
        params = urlencode( queryParams )
        url = YAHOO_SEARCH_URL % "%2B".join(map( quote, terms) ) +"?"+ params
        print url
        try:
            result = eval( self.r.open(url).read().replace("\\/", "/" ))
            return result['ysearchresponse']
        except URLError:
            return ""

    @staticmethod
    def getSearchResults(query_result):
        """ returns a list of all search results returned by the given
            query result.
            @param[in] query_result     Result of the query
        """
        return [ YahooSearchResult(r) for r in query_result['resultset_web'] ] \
           if 'resultset_web' in query_result else []


    def getTagInfo(self, tag):
        """ @Override """
        return int( self.query(tag)['totalhits'] )


class YahooSearchResult(object):
    """ Perfom manipulations on yahoo search results """

    __slots__ = ('search_result')

    def __init__(self, search_result):
        """ @param[in] search_result ... search result to query """
        self.search_result = search_result

    def getKeywords(self):
        """ @returns the keywords for the given search_result """
        return self.search_result['keywords']['terms']

    def getPageContent(self):
        """ @returns the content of the found web page """
        return urlopen( self.search_result['url'] ).read()

    def getPageText(self):
        """ @returns the text of the found web page """
        try:
            return HtmlToText.getText( self.getPageContent() )
        except:
            return ""

class TestYahoo(object):
    """ tests the yahoo search API """

    SEARCH_QUERIES = {
        'energy': ( ('energy', 'coal'), ('energy', 'sustainable') ),
        'latex' : ( ('latex', 'bibtex'), ('latex', 'knutz') )
    }

    def __init__(self):
        self.y = Yahoo()

    @attr("remote")
    def testSearchCounts(self):
        for query, refinedQueries in self.SEARCH_QUERIES.iteritems():
            qCount = int(self.y.query( (query, ) )['totalhits'])

            for q in refinedQueries:
                print query, q, "**",qCount, int(self.y.query( q )['totalhits'])
                assert qCount > int(self.y.query( q )['totalhits'])
    
    @attr("remote")
    def testTagInfo(self):
        """ tests the tag info service """
        assert self.y.getTagInfo( ('weblyzard',)) > 10
        assert self.y.getTagInfo( ('a_query_which_should_not_appear_at_all', )) == 0

    @attr("remote")
    def testYahooSearchResult(self):
        """ tests the Yahoo Search Result objects """
        for resultSite in Yahoo.getSearchResults(self.y.query( ("linux", "firefox", ),  \
                            count=1, queryParams={'view':'keyterms', 'abstract': 'long'} )):

            print resultSite.search_result['keyterms']['terms']
            assert len( resultSite.getPageText() ) > len(resultSite.search_result['abstract'])
            assert 'http' in resultSite.search_result['url']

    @attr("remote")
    def testBorderlineYahooSearchResult(self):
        """ tests borderline cases such as empty search results """
        assert len( Yahoo.getSearchResults(self.y.query( ('ksdaf', 'sadfj93', 'kd9', ), count=10, queryParams={'view':'keyterms', 'abstract': 'long'}) ) ) == 0

    @attr("remote")
    def testMultiProcessingRetrieve(self):
        """ tests the multi processing capabilities of this module """
        from multiprocessing import Pool
        p = Pool(4)

        TEST_URLS = ['http://www.derstandard.at', 
                     'http://www.dilbert.com',
                     'http://www.wetter.at',
                     'http://www.wu.ac.at',
                     'http://www.ai.wu.ac.at',
                     'http://www.tuwien.ac.at',
                     'http://www.boku.ac.at',
                     'http://www.univie.ac.at',
                    ]
        # f=open("/tmp/aw", "w")
        for res in p.map( p_fetchWebPage, TEST_URLS ):
            # f.write(res)
            # f.write("\n-----\n\n\n")
            assert len(res) > 20
        # f.close()

    @attr("remote")
    def testFailingUrls(self):
        """ tests the module with URLs known to fail(!) """
        TEST_URLS = ['http://www.mfsa.com.mt/insguide/english/glossarysearch.jsp?letter=all',
                    ]
        for url in TEST_URLS:
            assert len( p_fetchWebPage(url).strip() ) > 0



def p_fetchWebPage(url):
    """ fetches the web page specified in the given yahoo result object
        @param[in] url the url to fetch

        @remarks
        helper function for the testMultiProcessing test
    """
    r = YahooSearchResult( {'url': url} )
    return r.getPageText()


if __name__ == '__main__':
    y = Yahoo()
    #print y.query( ("energy",) )
    #print y.query( ("energy", "coal") )
    #print y.query( ("d'alembert", "law") )
    r = y.query( ("linux", "python", ), count=5, queryParams={'view': 'keyterms', 'abstract': 'long'} )
    print "***", r
    for entry in r['resultset_web']:
        print entry.keys()
        print entry['keyterms']['terms']
        print entry['url']
        print entry['abstract']





'''
Created on Feb 3, 2012

@author: heinz-peterlang
'''

class ArgumentErrorMsg(object):
    def __init__(self, function):
        self.function = function
        self.error_msg = ""
     
    def __call__(self):
        return "<h1>%s</h1><font color='red'>%s</font><p/><pre>%s</pre>" % \
             (self.function.__name__, self.error_msg, self.function.__doc__)

class MissingArgumentErrorMsg(ArgumentErrorMsg):

    def __init__(self, function, argument):
        ArgumentErrorMsg.__init__(self, function)
        self.error_msg = "Missing argument: '%s'." % (argument) 
    
class UnknownArgumentErrorMsg(ArgumentErrorMsg):

    def __init__(self, function, argument):
        ArgumentErrorMsg.__init__(self, function)
        self.error_msg = "Unknown argument: '%s'." % (argument)    